#include "cg4Java_CgFuncJNI.h"
#include <assert.h>
#include <CG/cg.h>

/*
 * Class:     cgFuncJNI
 * Method:    cgCreateContext
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgCreateContext
  (JNIEnv *env, jobject)
{
	return (jint)cgCreateContext();
}

/*
 * Class:     cgFuncJNI
 * Method:    cgDestroyContext
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_cg4java_CgFuncJNI_cgDestroyContext
  (JNIEnv *env, jobject, jint context)
{
	cgDestroyContext( (CGcontext)context );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgIsContext
 * Signature: (I)Z
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgIsContext
  (JNIEnv *env, jobject, jint context)
{
	return (int)cgIsContext( (CGcontext)context );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetLastListing
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetLastListing
  (JNIEnv *env, jobject, jint context)
{
	return env->NewStringUTF( cgGetLastListing( (CGcontext)context ) );
}

const char** getStringArrayChars( JNIEnv* env, jobjectArray array )
{
	int length = env->GetArrayLength(array);
	const char** c_args = new const char*[length+1];
	c_args[length] = 0;
	for( int i = 0 ; i < length; i++ )
	{
		jstring string = (jstring)env->GetObjectArrayElement( array, i );
		c_args[i] = env->GetStringUTFChars( string, 0);
	}
	return c_args;
}

/*
 * Class:     cgFuncJNI
 * Method:    cgCreateProgram
 * Signature: (IILjava/lang/String;ILjava/lang/String;[Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgCreateProgram
  (JNIEnv *env, jobject, jint context, jint programType, jstring program, jint profile, jstring entry, jobjectArray args)
{
	const char** c_args = 0;
	if( args != 0 )  c_args = getStringArrayChars( env, args );
	return (jint)cgCreateProgram( (CGcontext)context, 
		(CGenum)programType, 
		env->GetStringUTFChars( program, 0 ), 
		(CGprofile)profile,
		env->GetStringUTFChars( entry, 0 ),
		0);
}

/*
 * Class:     cgFuncJNI
 * Method:    cgCreateProgramFromFile
 * Signature: (IILjava/lang/String;ILjava/lang/String;[Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgCreateProgramFromFile
  (JNIEnv *env, jobject, jint context, jint programType, jstring program, jint profile, jstring entry, jobjectArray args)
{
	const char** c_args = 0;
	if( args != 0 )  c_args = getStringArrayChars( env, args );
	return (int)cgCreateProgramFromFile( (CGcontext)context,
		(CGenum)programType,
		env->GetStringUTFChars( program, 0 ), 
		(CGprofile)profile,
		env->GetStringUTFChars( entry, 0 ),
		0);
}

/*
 * Class:     cgFuncJNI
 * Method:    cgCopyProgram
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgCopyProgram
  (JNIEnv *, jobject, jint program)
{
	return (jint)cgCopyProgram( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgDestroyProgram
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_cg4java_CgFuncJNI_cgDestroyProgram
  (JNIEnv *, jobject, jint program)
{
	return cgDestroyProgram( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetFirstProgram
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetFirstProgram
  (JNIEnv *, jobject, jint context)
{
	return (jint)cgGetFirstProgram( (CGcontext)context );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetNextProgram
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetNextProgram
  (JNIEnv *, jobject, jint program)
{
	return (jint)cgGetNextProgram( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetProgramContext
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetProgramContext
  (JNIEnv *, jobject, jint program)
{
	return (jint)cgGetProgramContext( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgIsProgram
 * Signature: (I)Z
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgIsProgram
  (JNIEnv *, jobject, jint program)
{
	return (jint)cgIsProgram( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgCompileProgram
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_cg4java_CgFuncJNI_cgCompileProgram
  (JNIEnv *, jobject, jint program)
{
	cgCompileProgram( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgIsProgramCompiled
 * Signature: (I)Z
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgIsProgramCompiled
  (JNIEnv *, jobject, jint program)
{
	return (jint)cgIsProgramCompiled( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetProgramString
 * Signature: (II)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetProgramString
  (JNIEnv *env, jobject, jint program, jint stype)
{
	return env->NewStringUTF( cgGetProgramString( (CGprogram)program, (CGenum)stype ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetProgramProfile
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetProgramProfile
  (JNIEnv *, jobject, jint program)
{
	return (jint)cgGetProgramProfile( (CGprogram)program );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetNamedParameter
 * Signature: (ILjava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetNamedParameter
  (JNIEnv *env, jobject, jint program, jstring name)
{
	return (jint)cgGetNamedParameter( (CGprogram)program, env->GetStringUTFChars( name, 0 ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetFirstParameter
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetFirstParameter
  (JNIEnv *, jobject, jint program, jint nspace)
{
	return (jint)cgGetFirstParameter( (CGprogram)program, (CGenum)nspace );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetNextParameter
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetNextParameter
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetNextParameter( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetFirstLeafParameter
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetFirstLeafParameter
  (JNIEnv *, jobject, jint program, jint nspace)
{
	return (jint)cgGetFirstLeafParameter( (CGprogram)program, (CGenum)nspace );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetNextLeafParameter
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetNextLeafParameter
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetNextLeafParameter( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetFirstStructParameter
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetFirstStructParameter
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetFirstStructParameter( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetFirstDependentParameter
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetFirstDependentParameter
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetFirstDependentParameter( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetArrayParameter
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetArrayParameter
  (JNIEnv *, jobject, jint param, jint index)
{
	return (jint)cgGetArrayParameter( (CGparameter)param, index );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetArrayDimension
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetArrayDimension
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetArrayDimension( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetArraySize
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetArraySize
  (JNIEnv *, jobject, jint param, jint dimension)
{
	return (jint)cgGetArraySize( (CGparameter)param, dimension );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterProgram
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterProgram
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterProgram( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgIsParameter
 * Signature: (I)Z
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgIsParameter
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgIsParameter( (CGparameter) param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterName
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetParameterName
  (JNIEnv *env, jobject, jint param)
{
	return env->NewStringUTF( cgGetParameterName( (CGparameter)param ) );
}
	
/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterType
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterType
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterType( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterSemantic
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetParameterSemantic
  (JNIEnv *env, jobject, jint parameter)
{
	return env->NewStringUTF( cgGetParameterSemantic( (CGparameter)parameter ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterResource
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterResource
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterResource( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterBaseResource
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterBaseResource
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterBaseResource( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterResourceIndex
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterResourceIndex
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterResourceIndex( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterVariability
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterVariability
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterVariability( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterDirection
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterDirection
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterDirection( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgIsParameterReferenced
 * Signature: (I)Z
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgIsParameterReferenced
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgIsParameterReferenced( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterValues
 * Signature: (III)[D
 */
JNIEXPORT jdoubleArray JNICALL Java_cg4java_CgFuncJNI_cgGetParameterValues
  (JNIEnv *, jobject, jint, jint, jint);

/*
 * Class:     cgFuncJNI
 * Method:    cgGetParameterOrdinalNumber
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetParameterOrdinalNumber
  (JNIEnv *, jobject, jint param)
{
	return (jint)cgGetParameterOrdinalNumber( (CGparameter)param );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetTypeString
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetTypeString
  (JNIEnv *env, jobject, jint type)
{
	return env->NewStringUTF( cgGetTypeString( (CGtype)type ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetType
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetType
  (JNIEnv *env, jobject, jstring type)
{
	return (jint)cgGetType( env->GetStringUTFChars( type, 0 ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetResourceString
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetResourceString
  (JNIEnv *env, jobject, jint resource)
{
	return env->NewStringUTF( cgGetResourceString( (CGresource)resource ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetResource
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetResource
  (JNIEnv *env, jobject, jstring resource)
{
	return (jint)cgGetResource( env->GetStringUTFChars( resource, 0 ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetProfileString
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetProfileString
  (JNIEnv *env, jobject, jint profile)
{
	return env->NewStringUTF( cgGetProfileString( (CGprofile)profile ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetProfile
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetProfile
  (JNIEnv *env, jobject, jstring profile)
{
	return (jint)cgGetProfile( env->GetStringUTFChars( profile, 0 ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetError
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetError
  (JNIEnv *, jobject)
{
	return (jint)cgGetError();
}

/*
 * Class:     cgFuncJNI
 * Method:    cgGetErrorString
 * Signature: (I)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_cg4java_CgFuncJNI_cgGetErrorString
  (JNIEnv *env, jobject, jint error)
{
	return env->NewStringUTF( cgGetErrorString( (CGerror)error ) );
}

/*
 * Class:     cgFuncJNI
 * Method:    cgSetErrorCallback
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_cg4java_CgFuncJNI_cgSetErrorCallback
  (JNIEnv *, jobject, jint);

/*
 * Class:     cgFuncJNI
 * Method:    cgGetErrorCallback
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_cg4java_CgFuncJNI_cgGetErrorCallback
  (JNIEnv *, jobject);
