package vsl;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Random;
import java.util.*;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import javax.swing.*;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;

import vsl.*;
import vsl.dataflow.*;
import vsl.lang.*;
import vsl.util.*;

/*
 * Klasse fr ein visuelles Vertex-Programm
 */
public class VertexProgram extends DataFlowProgram
{

  /*
   * Erzeugt ein Fragment-Programm aus einem DOM-Dokument.
   */
  public VertexProgram( Document document )
  {
    super( document );
  }

  /*
   * Erzeugt ein neues Vertex-Programm.
   */
  public VertexProgram( String name )
  {
    super( name, Struct.getStruct( "v_in_ARB_small" ), Struct.getStruct( "v_out_ARB_small" ) );
  }

  /*
   * Erzeugt ein neues Vertex-Programm unter Angabe der Ein- und Ausgabe-Structs.
   */
  public VertexProgram( String name, Struct inputStruct, Struct outputStruct )
  {
    super( name, inputStruct, outputStruct );
  }

  /*
   * Exportiert das Programm in ein DOM-Document.
   */
  public Element exportXML( Document document )
  {
    Element rootXML = document.createElement( "VertexProgram" );
    rootXML.setAttribute( "name", getName() );

    Element inputStructXML = document.createElement( "InputStruct" );
    inputStructXML.setAttribute( "name", inputStructInstanceName );
    inputStructXML.setAttribute( "type", inputStruct.getName() );
    rootXML.appendChild( inputStructXML );

    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      Element outputXML = document.createElement( "Output" );
      outputXML.setAttribute( "name", export.getName() );
      outputXML.setAttribute( "type", export.getType().toString() );
      rootXML.appendChild( outputXML );
    }
    for ( int i = 0; i < imports.getAnchorCount(); i++ )
    {
      DataFlowSourceAnchor imp = ( DataFlowSourceAnchor ) imports.getAnchor( i );
      Element inputXML = document.createElement( "Input" );
      inputXML.setAttribute( "name", imp.getName() );
      inputXML.setAttribute( "type", imp.getType().toString() );
      DataValue defaultValue = imp.getDefault();
      if ( imp.getType().getJavaType() != DataType.J_UNDEFINED )
      {
        inputXML.setAttribute( "default", imp.getDefault().toString() );
      }
      rootXML.appendChild( inputXML );
    }

    Element outputStructXML = document.createElement( "OutputStruct" );
    outputStructXML.setAttribute( "name", outputStructInstanceName );
    outputStructXML.setAttribute( "type", outputStruct.getName() );
    rootXML.appendChild( outputStructXML );

    for ( int i = 0; i < outputStructExportCount; i++ )
    {
      Element exportXML = document.createElement( "Export" );
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) outputStructExports.
          getAnchor( i );
      exportXML.setAttribute( "name", export.getName() );
      if ( export.isConnected() )
      {
        Element connectionXML = export.getEdge().exportXML( document );
        exportXML.appendChild( connectionXML );
        rootXML.appendChild( exportXML );
      }
    }

    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      Element exportXML = document.createElement( "Export" );
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      exportXML.setAttribute( "name", export.getName() );
      if ( export.isConnected() )
      {
        Element connectionXML = export.getEdge().exportXML( document );
        exportXML.appendChild( connectionXML );
        rootXML.appendChild( exportXML );
      }
    }
    ArrayList sortedNodes = nodes.getSortedList();
    for ( int i = 0; i < sortedNodes.size(); i++ )
    {
      DataFlowNode node = ( DataFlowNode ) sortedNodes.get( i );
      if ( node.getMaxDistance() != -1 )
      {
        Element componentXML = node.exportXML( document );
        rootXML.appendChild( componentXML );
      }
    }

    return rootXML;
  }

}