package vsl.dataflow;

import java.awt.*;
import java.awt.image.BufferedImage;

import org.w3c.dom.*;

import vsl.lang.DataValue;
import vsl.lang.DataType;


/**
 * Basisklasse fr Knotenein- und ausgnge. Werden hier auch Verankerungen
 * genannt, da sie die geometrischen Endpunkte einer Verbindung darstellen.
 */
public abstract class DataFlowAnchor extends DataFlowComponent
{

  /*
   * Mgliche Ausprgungen (Ein- und Ausgnge)
   */
  public static int DF_DESTINATION_ANCHOR = 1;
  public static int DF_SOURCE_ANCHOR = 2;

  /*
   * Referenz zum Knoten.1
   */
  protected DataFlowNode node = null;

  /*
   * Name
   */
  protected String name;

  /*
   * Datentyp
   */
  protected DataType type;
  protected String typeString;

  /*
   * Defaultwert (falls vorhanden)
   */
  protected DataValue defaultValue = null;

  /*
   * Eigentliche geometrische Position, an der die Kante einer Verbindung
   * endet.
   */
  protected Point anchorPoint;

  /*
   * Beschriftung
   */
  protected DataFlowLabel label;

  /*
   * Erzeugt aus dem DOM-Element element eine neue Verankerung.
   */
  public DataFlowAnchor( Element element, DataFlowNode node)
  {
    this.name = element.getAttribute("name");
    this.type = DataType.getDataType( element.getAttribute("type") );
    if( !element.getAttribute( "default" ).equals("") )
    {
      this.defaultValue = new DataValue( this.type, element.getAttribute("default") );
    }
    this.node = node;
    anchorInit();
  }

  /*
   * Erzeugt neuen Verankerungspunkt ohne Default Wert.
   */
  public DataFlowAnchor( String name, DataType type, DataFlowNode node)
  {
    this.name = name;
    this.type = type;
    this.node = node;
    anchorInit();
  }

  /*
   * Erzeugt neuen Verankerungspunkt mit Default Wert.
   */
  public DataFlowAnchor( String name, DataType type, DataValue defaultValue, DataFlowNode node)
  {
    this.name = name;
    this.type = type;
    this.node = node;
    this.defaultValue = defaultValue;
    anchorInit();
  }

  /*
   * Allgemeine Initialisierung.
   */
  public void anchorInit()
  {
    label = new DataFlowLabel( name );
    anchorPoint = new Point( 0, 0 );
    setSize( type.getIcon().getWidth(), type.getIcon().getHeight() );
  }

  /*
   * Gibt wahr zurck, falls Verbindung vorhanden.
   */
  public abstract boolean isConnected();

  /*
   * Gibt Verankerungstyp (Quelle oder Ziel) zurck.
   */
  public abstract int getAnchorType();

  public void paint( Graphics g )
  {
    BufferedImage image = type.getIcon();
    g.drawImage(image, 0, 0, image.getWidth(), image.getHeight(), this);
  }

  /*
   * Gibt wahr zurck, falls Default-Wert vorhanden.
   */
  public boolean hasDefault()
  {
    if( defaultValue == null )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /*
   * Setzt den Default-Wert.
   */
  public void setDefaultValue( DataValue defaultValue )
  {
    this.defaultValue = defaultValue;
  }

  /*
   * Gibt den Default-Wert zurck.
   */
  public DataValue getDefault()
  {
    return defaultValue;
  }

  /*
   * Beschriftungen anzeigen oder ausblenden!
   */
  public void setShowLabel( boolean showLabel )
  {
    if( showLabel ) label.setVisible(true);
    else label.setVisible(false);
  }

  /*
   * Manche Verankerungen (Import und Export) haben keinen Knoten.
   * Dieses kann hier abgefragt werden.
   */
  public boolean hasNode()
  {
    if ( node == null )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /*
   * Gibt den Knoten zurck falls vorhanden, ansonsten null.
   */
  public DataFlowNode getNode()
  {
    return node;
  }

  /*
   * Farbe der Verankerung (ist vom Datentyp abhngig).
   */
  public Color getColor()
  {
    return type.getColor();
  }

  /*
   * Gibt die eigentliche Verbindungsposition zurck.
   */
  public Point getAnchorPoint()
  {
    return anchorPoint;
  }

  /*
   * Gibt Beschriftung zurck.
   */
  public DataFlowLabel getLabel()
  {
    return label;
  }

  /*
   * Gibt Namen zurck.
   */
  public String getName()
  {
    return name;
  }

  /*
   * Gibt Typ zurck.
   */
  public DataType getType()
  {
    return type;
  }


  public boolean isSelectable()
  {
    return true;
  }

  public boolean isDraggable()
  {
    return false;
  }

  public int getComponentType()
  {
    return DF_ANCHOR;
  }

}