package vsl.dataflow;

import java.awt.*;
import java.awt.image.*;

import java.io.*;

import javax.imageio.ImageIO;

import org.w3c.dom.*;

import vsl.lang.*;
import vsl.dataflow.util.*;

/*
 * Ein konstanter Knoten.
 */
public class DataFlowConstantNode extends DataFlowNode
{

  DataValue value;

  protected static BufferedImage image;
  protected BufferedImage icon;

  static
  {
    try
    {
      image = ImageIO.read( new File( "images/block_rot_small.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  public DataFlowConstantNode( Element root )
  {
    String dataType = root.getAttribute("dataType");
    DataType type = DataType.getDataType(dataType);
    String valueString = root.getAttribute("value");
    this.value = new DataValue( type, valueString );
    initConstantNode();
  }

  public DataFlowConstantNode( DataValue value )
  {
    this.value = value;
    initConstantNode();
  }

  public void initConstantNode()
  {
    name = "constant";

    inputs = new DataFlowAnchorList();
    outputs = new DataFlowAnchorList();
    outputs.addAnchor( new DataFlowSourceAnchor( "c", value.getType(), value, this ) );
    hideLabels();
    setSize( image.getWidth(), image.getHeight() );
    try
    {
      icon = ImageIO.read( new File( "images/empty.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  public void paint( Graphics g )
  {
    g.drawImage( image, 0, 0, getWidth(), getHeight(), this );
    g.drawImage( icon, ( getWidth() - icon.getWidth() ) / 2,
                 ( getHeight() - icon.getHeight() ) / 2, icon.getWidth(), icon.getHeight(), this );
  }

  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Node" );
    root.setAttribute( "type", "constant" );
    root.setAttribute( "name", "constant" );
    root.setAttribute( "id", getId() );
    root.setAttribute( "dataType", value.getType().toString() );
    root.setAttribute( "value", value.toString() );
    root.setAttribute( "x", Integer.toString( getX() ) );
    root.setAttribute( "y", Integer.toString( getY() ) );

    return root;
  }

  public int getNodeType()
  {
    return DF_CONSTANT_NODE;
  }

}