package vsl.dataflow;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Random;
import java.util.*;

import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import javax.swing.*;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import vsl.lang.*;
import vsl.dataflow.util.*;

/*
 * Ein Datenflussdiagramm.
 */
public class DataFlowDiagram extends DataFlowComponent
{

  protected Random randomizer;

  protected DataType allowedImports[];

  protected DataFlowComponent selectedComponent = null;

  protected DataFlowNodeHashMap nodes;
  protected Vector edges;

  /*
   * Ein- und Ausgabe-Elemente.
   */
  protected DataFlowAnchorList exports = null;
  protected DataFlowAnchorList imports = null;

  /*
   * Name und Datei-name
   */
  protected String name;
  protected String filename = null;

  /*
   * Zugehriger Knoten, falls Teil eines anderen Diagramms.
   */
  private DataFlowDiagramNode parent = null;

  /*
   * Erzeugt aus einem DOM-Dokument ein neues Diagramm.
   */
  public DataFlowDiagram( Document document )
  {
    Element root = document.getDocumentElement();
    this.name = root.getAttribute( "name" );
    diagramInit();
    importDiagram( root );
  }

  /*
   * Erzeugt aus einem DOM-Dokument ein neues Diagramm.
   */
  public DataFlowDiagram( Element root )
  {
    this.name = root.getAttribute( "name" );
    diagramInit();
    importDiagram( root );
  }

  /*
   * DOM-Import
   */
  public void importDiagram( Element root )
  {

    /*
     * Inputs, Outputs und Knoten importieren.
     */
    NodeList rootChilds = root.getChildNodes();
    for ( int i = 0; i < rootChilds.getLength(); i++ )
    {
      if ( rootChilds.item( i ).getNodeType() == Element.ELEMENT_NODE )
      {
        Element element = ( Element ) ( rootChilds.item( i ) );
        if ( element.getTagName().equals( "Input" ) )
        {
          addImport( new DataFlowSourceAnchor( element, null ) );
        }
        else if ( element.getTagName().equals( "Output" ) )
        {
          addExport( new DataFlowDestinationAnchor( element, null ) );
        }
        else if ( element.getTagName().equals( "Node" ) )
        {
          String nodeType = element.getAttribute( "type" );
          int x = Integer.parseInt( element.getAttribute( "x" ) );
          int y = Integer.parseInt( element.getAttribute( "y" ) );
          String id = element.getAttribute( "id" );
          if ( nodeType.equals( "diagram" ) )
          {
            DataFlowDiagramNode compNode = new DataFlowDiagramNode( element );
            compNode.setId( id );
            compNode.setLocation( x, y );
            setNode( compNode );
          }
          if ( nodeType.equals( "function" ) )
          {
            DataFlowFunctionNode functionNode = new DataFlowFunctionNode( element );
            functionNode.setId( id );
            functionNode.setLocation( x, y );
            setNode( functionNode );
          }
          if ( nodeType.equals( "operator" ) )
          {
            DataFlowOperatorNode operatorNode = new DataFlowOperatorNode( element );
            operatorNode.setId( id );
            operatorNode.setLocation( x, y );
            setNode( operatorNode );
          }
          if ( nodeType.equals( "constructor" ) )
          {
            DataFlowConstructorNode constructorNode = new DataFlowConstructorNode( element );
            constructorNode.setId( id );
            constructorNode.setLocation( x, y );
            setNode( constructorNode );
          }
          if ( nodeType.equals( "state" ) )
          {
            DataFlowStateNode stateNode = new DataFlowStateNode( element );
            stateNode.setId( id );
            stateNode.setLocation( x, y );
            setNode( stateNode );
          }
          if ( nodeType.equals( "constant" ) )
          {
            DataFlowConstantNode constantNode = new DataFlowConstantNode( element );
            constantNode.setId( id );
            constantNode.setLocation( x, y );
            setNode( constantNode );
          }
        }
      }
    }
    /*
     * Kanten importieren.
     */
    for ( int i = 0; i < rootChilds.getLength(); i++ )
    {
      if ( rootChilds.item( i ).getNodeType() == Element.ELEMENT_NODE )
      {
        Element element = ( Element ) ( rootChilds.item( i ) );
        if ( element.getTagName().equals( "Export" ) )
        {
          NodeList childList = element.getElementsByTagName( "Connection" );
          if ( childList.getLength() > 0 )
          {
            Element connectionElement = ( Element ) childList.item( 0 );
            String edgeX = connectionElement.getAttribute( "x" );
            String sourceName = connectionElement.getAttribute( "source" );
            String sourceId = connectionElement.getAttribute( "sourceid" );
            String sourceOutput = connectionElement.getAttribute( "output" );
            DataFlowNode node = nodes.getNode( sourceName, sourceId );
            DataFlowSourceAnchor sourceAnchor = node.getOutputAnchor( sourceOutput );
            DataFlowDestinationAnchor destinationAnchor = getExport( element.getAttribute( "name" ) );
            DataFlowEdge edge = connect( sourceAnchor, destinationAnchor );
            if ( edge != null && !edgeX.equals("") )
            {
              edge.setVerticalLine( Float.parseFloat( edgeX ) );
            }
          }
          NodeList importList = element.getElementsByTagName( "Import" );
          if ( importList.getLength() > 0 )
          {
            Element importElement = ( Element ) importList.item( 0 );
            String edgeX = importElement.getAttribute( "x" );

            String importName = importElement.getAttribute( "name" );
            DataFlowSourceAnchor sourceAnchor = getImport( importName );
            DataFlowDestinationAnchor destinationAnchor = getExport( element.getAttribute( "name" ) );
            DataFlowEdge edge = connect( sourceAnchor, destinationAnchor );
            if ( edge != null && !edgeX.equals(""))
            {
              edge.setVerticalLine( Float.parseFloat( edgeX ) );
            }
          }

        }
        else if ( element.getTagName().equals( "Node" ) )
        {
          NodeList dataList = element.getChildNodes();
          for ( int j = 0; j < dataList.getLength(); j++ )
          {
            if ( dataList.item( j ).getNodeType() == Element.ELEMENT_NODE )
            {
              Element dataElement = ( Element ) dataList.item( j );
              if ( dataElement.getTagName().equals( "Data" ) )
              {
                NodeList childList = dataElement.getElementsByTagName( "Connection" );
                if ( childList.getLength() > 0 )
                {

                  Element connectionElement = ( Element ) childList.item( 0 );
                  String edgeX = connectionElement.getAttribute( "x" );

                  String sourceName = connectionElement.getAttribute( "source" );
                  String sourceId = connectionElement.getAttribute( "sourceid" );
                  String sourceOutput = connectionElement.getAttribute( "output" );

                  String destinationName = element.getAttribute( "name" );
                  String destinationId = element.getAttribute( "id" );
                  String destinationInput = dataElement.getAttribute( "name" );

                  DataFlowSourceAnchor sourceAnchor = nodes.getNode( sourceName, sourceId ).
                      getOutputAnchor( sourceOutput );
                  DataFlowDestinationAnchor destinationAnchor = nodes.getNode( destinationName,
                      destinationId ).getInputAnchor( destinationInput );
                  DataFlowEdge edge = connect( sourceAnchor, destinationAnchor );
                  if ( edge != null && !edgeX.equals("") )
                  {
                    edge.setVerticalLine( Float.parseFloat( edgeX ) );
                  }
                }
                NodeList importList = dataElement.getElementsByTagName( "Import" );
                if ( importList.getLength() > 0 )
                {
                  Element importElement = ( Element ) importList.item( 0 );
                  String edgeX = importElement.getAttribute( "x" );
                  String destinationName = element.getAttribute( "name" );
                  String destinationId = element.getAttribute( "id" );
                  String destinationInput = dataElement.getAttribute( "name" );

                  DataFlowNode node = nodes.getNode( destinationName, destinationId );
                  DataFlowDestinationAnchor destinationAnchor = node.getInputAnchor(
                      destinationInput );
                  String importName = importElement.getAttribute( "name" );
                  DataFlowSourceAnchor sourceAnchor = getImport( importName );
                  DataFlowEdge edge = connect( sourceAnchor, destinationAnchor );
                  if ( edge != null && !edgeX.equals("") )
                  {
                    edge.setVerticalLine( Float.parseFloat( edgeX ) );
                  }
                }
              }
            }
          }
        }
      }
    }
  }

  /*
   * Neues leeres Diagramm erzeugen.
   */
  public DataFlowDiagram( String name )
  {
    this.name = name;
    diagramInit();
  }

  /*
   * Allgemeine Initialisierung.
   */
  private void diagramInit()
  {
    setLayout( null );

    edges = new Vector();
    nodes = new DataFlowNodeHashMap( 100 );
    exports = new DataFlowAnchorList();
    imports = new DataFlowAnchorList();
    randomizer = new Random();
    setSize( 775, 575 );
  }

  /*
   * Zeichenroutine.
   */
  public void paint( Graphics g )
  {
    g.setColor( Color.WHITE );
    g.fillRect( 0, 0, getWidth()-1, getHeight() - 1 );
    g.setColor( Color.DARK_GRAY );
    g.drawRect( 0, 0, getWidth()-1, getHeight() - 1 );
    g.drawString( name, 20, 20 );
    for ( int i = 0; i < edges.size(); i++ )
    {
      ( ( DataFlowEdge ) ( edges.elementAt( i ) ) ).draw( g );
    }
    if ( selectedComponent != null )
    {
      markSelected( g );
    }
    super.paint( g );
  }

  /*
   * Kennzeichung eines markierten Objekts.
   */
  public void markSelected( Graphics g )
  {
    g.setColor( Color.GRAY );
    Rectangle b = selectedComponent.getBounds();
    g.drawRect( b.x - 4, b.y - 4, 2, 2 );
    g.fillRect( b.x - 4, b.y - 4, 2, 2 );
    g.drawRect( b.x + b.width + 2, b.y - 4, 2, 2 );
    g.fillRect( b.x + b.width + 2, b.y - 4, 2, 2 );
    g.drawRect( b.x - 4, b.y + b.height + 2, 2, 2 );
    g.fillRect( b.x - 4, b.y + b.height + 2, 2, 2 );
    g.drawRect( b.x + b.width + 2, b.y + b.height + 2, 2, 2 );
    g.fillRect( b.x + b.width + 2, b.y + b.height + 2, 2, 2 );
  }

  /*
   * Dateinamen setzen
   */
  public void setFilename( String filename )
  {
    this.filename = filename;
  }

  /*
   * Gibt Dateinamen zurck.
   */
  public String getFilename()
  {
    return filename;
  }

  /*
   * bergeordneten Knoten setzen.
   */
  public void setDiagramNode( DataFlowDiagramNode diagramNode )
  {
    parent = diagramNode;
  }

  /*
   * Gibt bergeordneten Knoten zurck.
   */
  public DataFlowDiagramNode getDiagramNode()
  {
    return parent;
  }

  /*
   * Ausgewhlte Komponente setzen.
   */
  public void setSelected( DataFlowComponent selected )
  {
    this.selectedComponent = selected;
  }

  /*
   * Knoten aus Diagramm entfernen.
   */
  public void removeNode( DataFlowNode node )
  {
    for ( int i = 0; i < node.getInputCount(); i++ )
    {
      DataFlowDestinationAnchor anchor = node.getInputAnchor( i );
      if ( anchor.isConnected() )
      {
        disconnect( anchor );
      }
      remove( anchor );
      remove( anchor.getLabel() );
    }
    for ( int i = 0; i < node.getOutputCount(); i++ )
    {
      DataFlowSourceAnchor anchor = node.getOutputAnchor( i );
      if ( anchor.isConnected() )
      {
        Vector anchorEdges = anchor.getEdges();
        while( anchorEdges.size() > 0 )
        {
          DataFlowEdge edge = ( DataFlowEdge ) anchorEdges.elementAt( 0 );
          DataFlowDestinationAnchor destination = edge.getDestination();
          disconnect( destination );
        }
      }

      remove( anchor );
      remove( anchor.getLabel() );
    }
    nodes.removeNode( node );
    remove( node );
  }

  /*
   * Kante aus Diagramm entfernen.
   */
  public void removeEdge( DataFlowEdge edge )
  {
    edge.getDestination().disconnect();
    remove( edge );
    edge.getSource().removeEdge( edge );
    edges.remove( edge );
  }

  /*
   * Knoten-Anschluss trennen.
   */
  public void disconnect( DataFlowDestinationAnchor destination )
  {
    if ( destination.isConnected() )
    {
      DataFlowEdge edge = destination.getEdge();
      destination.disconnect();
      remove( edge );
      edge.getSource().removeEdge( edge );
      edges.remove( edge );
    }
  }

  /*
   * Zwei Anschlsse verbinden.
   */
  public DataFlowEdge connect( DataFlowSourceAnchor source, DataFlowDestinationAnchor destination )
  {
    if ( !destination.isConnected() && source.getType().equals( destination.getType() ) )
    {
      DataFlowEdge edge = new DataFlowEdge( source, destination );
      edges.add( edge );
      add( edge );
      if ( detectCycle() )
      {
        destination.disconnect();
        edges.remove( edge );
        return null;
      }
      else
      {
        recalculateDistances();
        return edge;
      }
    }
    return null;
  }

  /*
   * Neue Eingabe.
   */
  public void addImport( DataFlowSourceAnchor imp )
  {
    imports.addAnchor( imp );
    add( imp.getLabel() );
    add( imp );
    updateImportLocations();
  }

  /*
   * Eingaben neu anordnen.
   */
  public void updateImportLocations()
  {
    double importDistance = ( double ) getHeight() / ( double ) ( imports.getAnchorCount() + 1 );
    for ( int i = 0; i < imports.getAnchorCount(); i++ )
    {
      DataFlowAnchor imp = imports.getAnchor( i );
      imp.setLocation( 1, ( int ) ( importDistance * ( double ) ( i + 1 ) ) - imp.getHeight() / 2 );
    }
  }

  /*
   * Neue Ausgabe.
   */
  public void addExport( DataFlowDestinationAnchor export )
  {
    exports.addAnchor( export );
    add( export.getLabel() );
    add( export );
    updateExportLocations();
  }

  /*
   * Ausgaben neu sortieren.
   */
  public void updateExportLocations()
  {
    double exportDistance = ( double ) getHeight() / ( double ) ( exports.getAnchorCount() + 1 );
    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowAnchor export = exports.getAnchor( i );
      export.setLocation( getWidth() - export.getWidth() - 1,
                          ( int ) ( exportDistance * ( double ) ( i + 1 ) ) -
                          export.getHeight() / 2 );
    }
  }

  /*
   * Knoten in Diagramm einfgen.
   */
  public void addNode( DataFlowNode node )
  {
    while ( nodes.containsNode( node.getName(), node.getId() ) )
    {
      node.setId( Integer.toString( randomizer.nextInt( 100 ) ) );
    }
    node.setLocation( getWidth() / 2 - node.getWidth() / 2, getHeight() / 2 - node.getHeight() / 2 );
    setNode( node );
  }

  private void setNode( DataFlowNode node )
  {
    nodes.putNode( node );
    int n = node.getInputCount();
    for ( int i = 0; i < n; i++ )
    {
      DataFlowAnchor anchor = node.getInputAnchor( i );
      add( anchor.getLabel() );
      add( anchor );
    }
    n = node.getOutputCount();
    for ( int i = 0; i < n; i++ )
    {
      DataFlowAnchor anchor = node.getOutputAnchor( i );
      add( anchor.getLabel() );
      add( anchor );
    }
    add( node );
  }

  /*
   * Endeckt Zyklen innerhalb des Graphen.
   */
  public boolean detectCycle()
  {
    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      if ( export.isConnected() )
      {
        if ( export.getEdge().getSource().hasNode() )
        {
          if ( export.getEdge().getSource().getNode().detectCycle() )
          {
            return true;
          }
        }
      }
    }
    return false;
  }

  /*
   * Berechnet die maximalen Entfernungen der Knoten zu einer Ausgabe.
   */
  public void recalculateDistances()
  {
    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      if ( export.isConnected() )
      {
        if ( export.getEdge().getSource().hasNode() )
        {
          export.getEdge().getSource().getNode().calculateMaxDistance( 1 );
        }
      }
    }
  }

  /*
   * Gibt Informationen des Diagramms als Text in die Konsole aus.
   */
  public void print()
  {
    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      if ( export.isConnected() )
      {
        if ( export.getEdge().getSource().hasNode() )
        {
          export.getEdge().getSource().getNode().print();
        }
      }
    }
  }

  /*
   * Exportiert das Diagramm als DOM-Document.
   */
  public Document getXMLDocument()
  {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    try
    {
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.newDocument();
      document.appendChild( exportXML( document ) );
      return document;
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
    return null;
  }

  /*
   * Gibt das Diagramm als XML in der Konsole aus.
   */
  public void printXMLDocument()
  {
    try
    {
      FileOutputStream xmlOut = new FileOutputStream( new File( "xml/xmlOutput.xml" ) );
      TransformerFactory tffactory = TransformerFactory.newInstance();
      Transformer transformer = tffactory.newTransformer();

      DOMSource source = new DOMSource( getXMLDocument() );
      StreamResult result = new StreamResult( System.out );
      transformer.transform( source, result );
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }

  /*
   * Exportiert das Diagramm als DOM und fgt es in das Dokument ein.
   */
  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Diagram" );
    root.setAttribute( "name", getName() );
    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      Element outputXML = document.createElement( "Output" );
      outputXML.setAttribute( "name", export.getName() );
      outputXML.setAttribute( "type", export.getType().toString() );
      root.appendChild( outputXML );
    }
    for ( int i = 0; i < imports.getAnchorCount(); i++ )
    {
      DataFlowSourceAnchor imp = ( DataFlowSourceAnchor ) imports.getAnchor( i );
      Element inputXML = document.createElement( "Input" );
      inputXML.setAttribute( "name", imp.getName() );
      inputXML.setAttribute( "type", imp.getType().toString() );
      if( imp.getType().getJavaType() != DataType.J_UNDEFINED )
      {
        inputXML.setAttribute( "default", imp.getDefault().toString() );
      }
      root.appendChild( inputXML );
    }
    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      Element exportXML = document.createElement( "Export" );
      exportXML.setAttribute( "name", exports.getAnchor( i ).getName() );
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      if ( export.isConnected() )
      {
        Element connectionXML = export.getEdge().exportXML( document );
        exportXML.appendChild( connectionXML );
        root.appendChild( exportXML );
      }
    }
    ArrayList sortedNodes = nodes.getSortedList();
    for ( int i = 0; i < sortedNodes.size(); i++ )
    {
      DataFlowNode node = ( DataFlowNode ) sortedNodes.get( i );
      if ( node.getMaxDistance() != -1 )
      {
        Element componentXML = node.exportXML( document );
        root.appendChild( componentXML );
      }
    }
    return root;
  }

  /*
   * Anzahl der Eingaben.
   */
  public int getImportCount()
  {
    return imports.getAnchorCount();
  }

  /*
   * Gibt Eingabe i zurck.
   */
  public DataFlowAnchor getImport( int i )
  {
    return imports.getAnchor( i );
  }

  /*
   * Gibt Eingabe mit Namen name zurck.
   */
  public DataFlowSourceAnchor getImport( String name )
  {
    if ( imports.containsAnchor( name ) )
    {
      return ( DataFlowSourceAnchor ) imports.getAnchor( name );
    }
    else
    {
      return null;
    }
  }

  /*
   * Gibt Anzahl der Ausgaben zurck.
   */
  public int getExportCount()
  {
    return exports.getAnchorCount();
  }

  /*
   * Gibt Ausgabe i zurck.
   */
  public DataFlowAnchor getExport( int i )
  {
    return exports.getAnchor( i );
  }

  /*
   * Gibt Ausgabe mit Namen name zurck.
   */
  public DataFlowDestinationAnchor getExport( String name )
  {
    if ( exports.containsAnchor( name ) )
    {
      return ( DataFlowDestinationAnchor ) exports.getAnchor( name );
    }
    else
    {
      return null;
    }
  }

  /*
   * Gibt den Namen zurck.
   */
  public String getName()
  {
    return name;
  }

  /*
   * Setzt den Namen des Diagramms.
   */
  public void setName( String name )
  {
    this.name = name;
  }

  /*
   * Setzt die Gre des Diagramms.
   */
  public void setSize( Dimension d )
  {
    setSize( d.width, d.height );
  }

  /*
   * Setzt die Gre des Diagramms.
   */
  public void setSize( int w, int h )
  {
    super.setSize( w, h );
    updateImportLocations();
    updateExportLocations();

  }

  public boolean isSelectable()
  {
    return true;
  }

  public boolean isDraggable()
  {
    return true;
  }

  /*
   * Gibt den Komponenten-Typ zurck.
   */
  public int getComponentType()
  {
    return DF_DIAGRAM;
  }
}