package vsl.dataflow;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;

import java.io.*;

import javax.imageio.ImageIO;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import org.w3c.dom.*;

import vsl.dataflow.util.*;

/*
 * node that itself is a dataflow diagram
 */
public class DataFlowDiagramNode extends DataFlowNode
{

  protected static BufferedImage image;
  protected static BufferedImage icon;

  static
  {
    try
    {
      icon = ImageIO.read( new File( "images/diagram.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  static
  {
    try
    {
      image = ImageIO.read( new File( "images/block_big.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  DataFlowDiagram diagram;

  public DataFlowDiagramNode( Element nodeElement )
  {
    NodeList nodeList = nodeElement.getChildNodes();
    for ( int i = 0; i < nodeList.getLength(); i++ )
    {
      if ( nodeList.item( i ).getNodeType() == Element.ELEMENT_NODE )
      {
        Element element = ( Element ) ( nodeList.item( i ) );
        if ( element.getTagName().equals( "Diagram" ) )
        {
          diagram = new DataFlowDiagram( element );
        }
      }
    }
    initDiagramNode();
  }

  public DataFlowDiagramNode( DataFlowDiagram diagram )
  {
    this.diagram = diagram;
    initDiagramNode();
  }

  public void initDiagramNode()
  {
    name = diagram.getName();
    diagram.setDiagramNode( this );
    id = "00";
    inputs = new DataFlowAnchorList();
    for ( int i = 0; i < diagram.getImportCount(); i++ )
    {
      DataFlowAnchor imp = diagram.getImport( i );
      inputs.addAnchor( new DataFlowDestinationAnchor( imp.getName(), imp.getType(), imp.getDefault(), this ) );
    }
    outputs = new DataFlowAnchorList();
    for ( int i = 0; i < diagram.getExportCount(); i++ )
    {
      DataFlowAnchor imp = diagram.getExport( i );
      outputs.addAnchor( new DataFlowSourceAnchor( imp.getName(), imp.getType(), imp.getDefault(), this ) );
    }

    showLabels();
    setSize( image.getWidth(), image.getHeight() );
  }

  public void paint( Graphics g )
  {
    g.drawImage( image, 0, 0, getWidth(), getHeight(), this );
    g.drawImage( icon, ( getWidth() - icon.getWidth() ) / 2, ( getHeight() - icon.getHeight() ) / 2,
                 icon.getWidth(), icon.getHeight(), this );
    Font font = new Font( "SansSerif", Font.PLAIN, 10 );
    g.setFont( font );
    String title = name;
    Rectangle2D rectangle = g.getFontMetrics( font ).getStringBounds( title, g );
    int fontHeight = ( int ) rectangle.getHeight();
    g.drawString( title, 5, fontHeight );
  }

  // diagram changed?
  public void update()
  {
    if ( inputs.getAnchorCount() < diagram.getImportCount() )
    {
      for ( int i = 0; i < diagram.getImportCount(); i++ )
      {
        DataFlowAnchor imp = diagram.getImport( i );
        if ( !inputs.containsAnchor( imp.getName() ) )
        {
          DataFlowDestinationAnchor input = new DataFlowDestinationAnchor( imp.getName(),
              imp.getType(), imp.getDefault(), this );
          inputs.addAnchor( input );
          getParent().add( input );
          getParent().add( input.getLabel() );
        }
      }
    }
    if ( outputs.getAnchorCount() < diagram.getExportCount() )
    {
      for ( int i = 0; i < diagram.getExportCount(); i++ )
      {
        DataFlowAnchor export = diagram.getExport( i );
        if ( !outputs.containsAnchor( export.getName() ) )
        {
          DataFlowSourceAnchor output = new DataFlowSourceAnchor( export.getName(), export.getType(),
              export.getDefault(), this );
          outputs.addAnchor( output );
          getParent().add( output );
          getParent().add( output.getLabel() );
        }
      }
    }
    updateAnchorLocations();
    getParent().repaint();
  }

  public DataFlowDiagram getDiagram()
  {
    return diagram;
  }

  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Node" );
    root.setAttribute( "type", "diagram" );
    root.setAttribute( "name", getName() );
    root.setAttribute( "id", getId() );
    root.setAttribute( "x", Integer.toString( getX() ) );
    root.setAttribute( "y", Integer.toString( getY() ) );

    for ( int i = 0; i < inputs.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor input = ( DataFlowDestinationAnchor ) inputs.getAnchor( i );
      Element dataElement = document.createElement( "Data" );
      dataElement.setAttribute( "name", input.getName() );
      root.appendChild( dataElement );
      if ( input.isConnected() )
      {
        DataFlowEdge edge = input.getEdge();
        Element inputElement = edge.exportXML( document );
        dataElement.appendChild( inputElement );
      }
      else
      {
        Element defaultElement = document.createElement( "Default" );
        dataElement.appendChild( defaultElement );
        defaultElement.setAttribute("value", input.getDefault().toString());
      }
    }
    Element diagramXML = diagram.exportXML( document );
    root.appendChild( diagramXML );
    return root;
  }

  public int getNodeType()
  {
    return DF_DIAGRAM_NODE;
  }

}