package vsl.dataflow;

import java.awt.*;

import java.util.Vector;

import org.w3c.dom.*;

/*
 * Verbindung zwischen zwei Anschlssen zu. Eine Verbing besteht aus
 * zwei horizontalen Linien und einer vertikalen Linie. Ausserdem enthlt sie
 * noch eine Komponente mit der die Position der vertikalen Linie kontrolliert
 * werden kann.
 */
public class DataFlowEdge extends DataFlowComponent
{

  private boolean doubleLine = true;

  /*
   * Position der vert. Linie
   */
  private float verticalLine = 0.5f;

  /*
   * Quell- und Ziel-Anschluss
   */
  protected DataFlowSourceAnchor source;
  protected DataFlowDestinationAnchor destination;

  /*
   * Start- und Endpunkte der Linien
   */
  Point p0, p1, p2, p3;

  /*
   * Konstruktor
   */
  public DataFlowEdge( DataFlowSourceAnchor source, DataFlowDestinationAnchor destination )
  {
    this.source = source;
    this.destination = destination;
    source.addEdge( this );
    destination.setEdge( this );
    if( doubleLine == false ) setSize( 5, 5 );
    else setSize( 6, 6 );
    p0 = source.getAnchorPoint();
    p1 = new Point();
    p2 = new Point();
    p3 = destination.getAnchorPoint();
    update();
  }

  /*
   * Zeichnet das Verschiebe-Kstchen
   */
  public void paint( Graphics g )
  {
    g.setColor( getColor() );
    g.drawRect( 0, 0, getWidth(), getHeight() );
    g.fillRect( 0, 0, getWidth(), getHeight() );
  }

  /*
   * Zeichnet die Liniensegemente
   */
  public void draw( Graphics g )
  {
    g.setColor( source.getColor() );
    g.drawLine( p0.x, p0.y, p1.x, p1.y );
    g.drawLine( p1.x, p1.y, p2.x, p2.y );
    g.drawLine( p2.x, p2.y, p3.x, p3.y );

    if( doubleLine == true )
    {
      g.drawLine( p0.x, p0.y+1, p1.x, p1.y+1 );
      g.drawLine( p1.x-1, p1.y, p2.x-1, p2.y );
      g.drawLine( p2.x, p2.y-1, p3.x, p3.y-1 );
    }

    Polygon triangle = new Polygon();
    triangle.addPoint( p3.x, p3.y );
    triangle.addPoint( p3.x - 5, p3.y + 3 );
    triangle.addPoint( p3.x - 5, p3.y - 3 );
    g.drawPolygon( triangle );
    g.fillPolygon( triangle );
  }

  /*
   * Gibt die Linienfarbe zurck.
   */
  public Color getColor()
  {
    return source.getColor();
  }

  /*
   * Setzt die Position der Positionierungs-Komponente.
   */
  public void setLocation( int x, int y )
  {
    super.setLocation( x, getY() );
    verticalLine = ( float ) ( x+getWidth()/2 - getSource().getAnchorPoint().getX() ) /
        ( float ) ( getDestination().getAnchorPoint().getX() -
                    getSource().getAnchorPoint().getX() );
    p1.x = p0.x + ( int ) ( (( float ) ( p3.x - p0.x )) * verticalLine );
    p2.x = p1.x;
  }

  public void setVerticalLine( float value )
  {
    this.verticalLine = value;
    update();
  }

  public DataFlowSourceAnchor getSource()
  {
    return source;
  }

  public void setSource( DataFlowSourceAnchor source )
  {
    this.source = source;
  }

  public DataFlowDestinationAnchor getDestination()
  {
    return destination;
  }

  public void setDestination( DataFlowDestinationAnchor destination )
  {
    this.destination = destination;
  }

  // update is called when an anchor has moved
  public void update()
  {
    p0 = source.getAnchorPoint();
    p3 = destination.getAnchorPoint();
    if ( p0 != null && p3 != null )
    {
      p1.x = p0.x + ( int ) ( ( float ) ( p3.x - p0.x ) * verticalLine );
      p1.y = p0.y;
      p2.x = p1.x;
      p2.y = p3.y;
      super.setLocation( p1.x - getWidth() / 2, ( p0.y + p3.y ) / 2 - getHeight()/2 );
    }
  }

  // exports itself as DOM element
  public Element exportXML( Document document )
  {
    Element root = null;
    if ( source.hasNode() )
    {
      root = document.createElement( "Connection" );
      root.setAttribute( "source", source.getNode().getName() );
      root.setAttribute( "sourceid", source.getNode().getId() );
      root.setAttribute( "output", source.getName() );
      root.setAttribute( "x", Float.toString( verticalLine ) );
    }
    else
    {
      root = document.createElement( "Import" );
      root.setAttribute( "name", source.getName() );
      root.setAttribute( "x", Float.toString( verticalLine ) );
    }
    return root;
  }

  public boolean isDraggable()
  {
    return true;
  }

  public boolean isSelectable()
  {
    return true;
  }

  public int getComponentType()
  {
    return DataFlowComponent.DF_EDGE;
  }

}