package vsl.dataflow;

import java.io.*;
import javax.imageio.*;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

import org.w3c.dom.*;

import vsl.lang.*;
import vsl.dataflow.util.*;

/*
 * node for OpenGL state variables
 */
public class DataFlowStateNode extends DataFlowNode
{

  // corresponding state variable
  StateVariable variable;

  protected static BufferedImage image;
  protected BufferedImage icon;

  static
  {
    try
    {
      image = ImageIO.read( new File( "images/block_small.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  public DataFlowStateNode( Element element )
  {
    this.variable = StateVariable.getVariable(element.getAttribute("name"));
    initStateNode();
  }

  public DataFlowStateNode( StateVariable variable )
  {
    this.variable = variable;
    initStateNode();
  }

  public void initStateNode()
  {
    this.id = "00";
    name = variable.getName();
    icon = variable.getIcon();

    inputs = new DataFlowAnchorList();

    outputs = new DataFlowAnchorList();
    DataFlowSourceAnchor anchor = new DataFlowSourceAnchor( variable.getShortName(),
        variable.getType(), this );
    outputs.addAnchor( anchor );
    showLabels();
    setSize( image.getWidth(), image.getHeight() );
  }

  public void paint( Graphics g )
  {
    g.drawImage( image, 0, 0, getWidth(), getHeight(), this );
    g.drawImage( icon, ( getWidth() - icon.getWidth() ) / 2,
                 ( getHeight() - icon.getHeight() ) / 2, icon.getWidth(), icon.getHeight(), this );
  }

  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Node" );
    root.setAttribute( "type", "state" );
    root.setAttribute( "name", getName() );
    root.setAttribute( "value", variable.getValue() );

    root.setAttribute( "id", getId() );
    root.setAttribute( "x", Integer.toString( getX() ) );
    root.setAttribute( "y", Integer.toString( getY() ) );
    return root;
  }

  public int getNodeType()
  {
    return DF_STATE_NODE;
  }

}