package vsl.gui;

import cg4java.*;
import gl4java.awt.*;
import gl4java.utils.glut.*;
import gl4java.utils.textures.*;

import java.awt.event.*;
import java.lang.Math.*;

import org.w3c.dom.*;

import vsl.util.*;
import vsl.lang.*;

/*
 * Diese Klasse stellt eine Testumgebung fr Vertex- und Fragment-Shader zur
 * Verfgung. Von aussen knnen Texturen und CubeMaps in die Textureinheiten
 * geladen. Die Vertex- und Fragment-Programme werden als DOM-Dokument bergeben
 */
public class PreviewCanvas extends GLAnimCanvas implements CgEnum, CgGLEnum, MouseListener,
    MouseMotionListener, KeyListener
{

  /*
   * OpenGL-Konstanten der Cube-Map-Seiten
   */
  static int cubefaces[] =
      {
      GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT,
      GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT,
      GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT,
      GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT,
      GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT,
      GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT,
  };

  /*
   * OpenGL-Konstanten zur Angabe der Textureinheit
   */
  static int units[] =
      {
      GL_TEXTURE0_ARB,
      GL_TEXTURE1_ARB,
      GL_TEXTURE2_ARB,
      GL_TEXTURE3_ARB,
      GL_TEXTURE4_ARB,
      GL_TEXTURE5_ARB,
      GL_TEXTURE6_ARB,
      GL_TEXTURE7_ARB
  };

  /*
   * Cg4Java Klassen
   */
  static CgFunc cg = new CgFuncJNI();
  static CgGLFunc cgGL = new CgGLFuncJNI();
  private int cgContext;

  /*
   * GLUT-Funktionalitt
   */
  static GLUTFunc glut;

  /*
   * Vertex- und Fragment-Programm
   */
  private int vertexProgram = 0;
  private String vertexProgramString;
  private boolean vertex = false;

  private int fragmentProgram = 0;
  private String fragmentProgramString;
  private boolean fragment = false;

  /*
   * Transformer fr die Transformation des DOM-Dokuments nach Cg-Source
   */
  private ShaderTransformer transformer;

  /*
   * maximal verfgbare Textureinheiten
   */
  private int glMaxTextures = 0;

  /*
   * Objekt zum Laden von PNG-Texturen
   */
  PngTextureLoader texLoader;

  /*
   * Beim laden einer Cube-Map wird ein Hintergrundbild angezeigt
   */
  private int[] background_texture = new int[1];
  private boolean background = false;

  /*
   * geladene Texturen
   */
  private int[][] textures;
  private int[] textureType;
  private boolean[] textureLoaded;


  /*
   * Hilfsvariablen fr das setzen neuer Cube-Maps von auen
   */
  private final int maxNewCubeMapCount = 8;
  private String newCubeMapFilenames[][] = new String[8][];
  private int newCubeMapTextureUnit[] = new int[8];
  private int newCubeMapCount = 0;

  /*
   * Hilfsvariablen fr das Setzen neuer Programme von auen
   */
  private final int maxNewDocumentCount = 8;
  private Document newDocuments[] = new Document[8];
  private int newDocumentCount = 0;

  /*
   * Hilfsvariablen fr das Setzen neuer Texturen von auen
   */
  private final int maxNewTexturesCount = 8;
  private String newTexturesFilename[] = new String[8];
  private int newTexturesUnit[] = new int[8];
  private int newTexturesCount = 0;

  /*
   * Zugriffsperren
   */
  private boolean programsLock = false;
  private boolean textureLock = false;

  /*
   * aktuelle DOM-Dokumente der Programme
   */
  private Document vertexDocument;
  private Document fragmentDocument;

  /*
   * Ist Vertex-Programm aktiviert?
   */
  private boolean vertexEnabled;

  /*
   * Maus-Kontrolle des Canvas
   */
  private float rotX = 0;
  private float rotY = 0;
  private float pressedX;
  private float pressedY;
  private float pressedRotX = 0.0f;
  private float pressedRotY = 0.0f;

  /*
   * Rotationen des Testobjekts und der Lichtquelle
   */
  private float objectRotation = 0;
  private float lightRotation = 0;
  private float lightRotOffset = 1.6f;

  /*
   * Welches Objekt wird aktuell angezeigt?
   */
  private int objectIndex = 1;


  /*
   * Materialeigenschaften des Testobjekts
   */
  float diffuseMaterial0[] =
      {
      1.0f, 1.0f, 1.0f, 1.0f};
  float specularMaterial0[] =
      {
      1.0f, 1.0f, 1.0f, 1.0f};
  int shininessMaterial0 = 16;

  float light_position[] = new float[]
      {
      1.0f, 0.0f, 3.0f, 1.0f};

  /*
   * Konstruktor
   */
  public PreviewCanvas( int width, int height )
  {
    super( width, height );
    glut = new GLUTFuncLightImpl( gl, glu );

    addKeyListener( this );
    addMouseListener( this );
    addMouseMotionListener( this );
  }

  public void preinit()
  {
  }

  public void setSize( int width, int height )
  {
    reshape( width, height);
  }

  /*
   * Allgemeine Initialisierung
   */
  public void init()
  {
    texLoader = new PngTextureLoader( gl, glu );

    initExtensions();
    initTextures();
    initLight();
    initCg();
    transformer = new ShaderTransformer( "xml/transform.xsl" );

    gl.glEnable( GL_DEPTH_TEST );

    gl.glMatrixMode( GL_PROJECTION );
    glu.gluPerspective( 40.0, 1.0, 1.0, 10.0 );
    gl.glMatrixMode( GL_MODELVIEW );
    glu.gluLookAt( 0.0, 0.0, 7.5,
                   0.0, 0.0, 0.0,
                   0.0, 1.0, 0.0 );
    start();
  }

  /*
   * Cg initialisieren
   */
  void initCg()
  {
    cgContext = cg.cgCreateContext();
    if ( cgGL.cgGLIsProfileSupported( CG_PROFILE_ARBVP1 ) == CG_TRUE )
    {
      System.out.println( "ARB_VERTEX_PROGRAM supported." );
    }
    else
    {
      System.out.println( "ARB_VERTEX_PROGRAM not supported." );
    }
    if ( cgGL.cgGLIsProfileSupported( CG_PROFILE_ARBFP1 ) == CG_TRUE )
    {
      System.out.println( "ARB_FRAGMENT_PROGRAM supported." );
    }
    else
    {
      System.out.println( "ARB_FRAGMENT_PROGRAM not supported." );
    }
  }

  /*
   * Multitexture-Extension prfen und initialisieren
   */
  void initExtensions()
  {
    String extensions = gl.glGetString( GL_EXTENSIONS );
    System.out.println( extensions );
    if ( extensions.indexOf( "GL_ARB_multitexture" ) != -1 )
    {
      int[] maxTex = new int[1];
      gl.glGetIntegerv( GL_MAX_TEXTURE_UNITS_ARB, maxTex );
      glMaxTextures = maxTex[0];
    }
    else
    {
      glMaxTextures = 1;
    }
  }

  /*
   * Texturvariablen initialisieren
   */
  void initTextures()
  {
    textures = new int[glMaxTextures][1];
    textureLoaded = new boolean[glMaxTextures];
    textureType = new int[glMaxTextures];
    for ( int i = 0; i < glMaxTextures; i++ )
    {
      textureLoaded[i] = false;
    }
  }

  /*
   * Lichtquellen initialisieren
   */
  public void initLight()
  {
    float ambientLight0[] =
        {
        0.1f, 0.1f, 0.1f, 1.0f};
    float diffuseLight0[] =
        {
        0.7f, 0.7f, 0.7f, 1.0f};
    float specularLight0[] =
        {
        1.0f, 1.0f, 1.0f, 1.0f};
    gl.glLightfv( GL_LIGHT0, GL_AMBIENT, ambientLight0 );
    gl.glLightfv( GL_LIGHT0, GL_DIFFUSE, diffuseLight0 );
    gl.glLightfv( GL_LIGHT0, GL_SPECULAR, specularLight0 );

    gl.glEnable( GL_LIGHT0 );
    gl.glEnable( GL_LIGHTING );
  }

  /*
   * Sperrt den Zugriff um auf die Variablen fr neue Programme zuzugreifen
   */
  private synchronized void lockPrograms()
  {
    while ( programsLock )
    {
      ;
    }
    programsLock = true;
  }

  /*
   * Gibt die Sperre wieder frei
   */
  private void releasePrograms()
  {
    programsLock = false;
  }

  /*
   * Falls neue Programme von aussen gesetzt wurden, werden diese nun
   * geladen und aktiviert.
   */
  private void newPrograms()
  {
    lockPrograms();
    for ( int i = 0; i < newDocumentCount; i++ )
    {
      Element root = newDocuments[i].getDocumentElement();
      if ( root.getTagName().equals( "VertexProgram" ) )
      {
        if ( vertexProgram != 0 )
        {
          cg.cgDestroyProgram( vertexProgram );
        }
        vertexProgramString = transformer.transform( newDocuments[i] );
        vertexProgram = cg.cgCreateProgram( cgContext, CG_SOURCE, vertexProgramString,
                                            CG_PROFILE_ARBVP1, "main", null );
        cg.cgCompileProgram( vertexProgram );
        cgGL.cgGLLoadProgram( vertexProgram );
        cgGL.cgGLBindProgram( vertexProgram );
        int error = cg.cgGetError();
        if ( error != CG_NO_ERROR )
        {
          System.err.println( "An Error occurred." );
          System.err.println( cg.cgGetErrorString( error ) );
          System.err.println( vertexProgramString );
        }
        else
        {
          vertexDocument = newDocuments[i];
          bindParameters( vertexProgram, vertexDocument );
          vertex = true;
          System.out.println( "Vertex-Program enabled." );
          System.out.println(cg.cgGetProgramString(vertexProgram, CG_COMPILED_PROGRAM ));
          System.out.println(cg.cgGetProgramString(vertexProgram, CG_PROGRAM_SOURCE ));
        }
      }
      else if ( root.getTagName().equals( "FragmentProgram" ) )
      {
        fragmentProgramString = transformer.transform( newDocuments[i] );
        fragmentProgram = cg.cgCreateProgram( cgContext, CG_SOURCE, fragmentProgramString,
                                              CG_PROFILE_ARBFP1, "main", null );
        cg.cgCompileProgram( fragmentProgram );
        cgGL.cgGLLoadProgram( fragmentProgram );
        cgGL.cgGLBindProgram( fragmentProgram );
        int error = cg.cgGetError();
        if ( error != CG_NO_ERROR )
        {
          System.err.println( "An Error occurred." );
          System.err.println( cg.cgGetErrorString( error ) );
          System.err.println( fragmentProgramString );
        }
        else
        {
          fragmentDocument = newDocuments[i];
          fragment = true;
          System.out.println( "Fragment-Program enabled." );
        }
      }
      newDocuments[i] = null;
    }
    newDocumentCount = 0;
    releasePrograms();
  }

  /*
   * Ein Programm von aussen setzen
   */
  public void setProgram( Document document )
  {
    lockPrograms();
    if ( newDocumentCount < maxNewDocumentCount )
    {
      newDocuments[newDocumentCount] = document;
      newDocumentCount++;
    }
    releasePrograms();
  }

  /*
   * Default-Werte der Programme binden
   */
  void bindParameters( int cgProgram, Document document )
  {
    Element root = document.getDocumentElement();
    NodeList list = root.getChildNodes();
    int n = list.getLength();
    for ( int i = 0; i < n; i++ )
    {
      Node node = list.item( i );
      if ( node.getNodeType() == Node.ELEMENT_NODE )
      {
        Element element = ( Element ) node;
        if ( element.getTagName().equals( "Input" ) )
        {
          String inputName = element.getAttribute( "name" );
          if ( inputName.equals( "t" ) )
          {
            break;
          }
          String inputType = element.getAttribute( "type" );
          String inputValue = element.getAttribute( "default" );
          if ( inputValue == "" || inputValue == null )
          {
            break;
          }
          DataType type = DataType.getDataType( inputType );
          DataValue value = new DataValue( type, inputValue );
          int param = cg.cgGetNamedParameter( cgProgram, inputName );
          float[] data = value.getFloatData();
          if ( data.length <= 4 )
          {
            switch ( data.length )
            {
              case 1:
                cgGL.cgGLSetParameter1f( param, data[0] );
                break;
              case 2:
                cgGL.cgGLSetParameter2fv( param, data );
                break;
              case 3:
                cgGL.cgGLSetParameter3fv( param, data );
                break;
              case 4:
                cgGL.cgGLSetParameter4fv( param, data );
            }
          }
        }
      }
    }
  }

  /*
   * Sperrt den Zugriff, um auf die Variablen fr neue Texturen zuzugreifen
   */
  private synchronized void lockNewTextures()
  {
    while ( textureLock )
    {
      ;
    }
    textureLock = true;
  }

  /*
   * Gibt Sperre wieder frei
   */
  private void releaseNewTextures()
  {
    textureLock = false;
  }

  /*
   * Gibt die Anzahl der verfgbaren Textureinheiten zurck.
   */
  public int getMaxTextures()
  {
    return glMaxTextures;
  }

  /*
   * Cube Map von auen setzen. Der String-Vektor enthlt die Dateinamen
   * der sechs verschiedenen Seiten.
   */
  public void setCubeMap( String[] filenames, int textureUnit )
  {
    lockNewTextures();
    if ( newCubeMapCount < maxNewCubeMapCount )
    {
      newCubeMapFilenames[newCubeMapCount] = filenames;
      newCubeMapTextureUnit[newCubeMapCount] = textureUnit;
      newCubeMapCount++;
    }
    releaseNewTextures();
  }

  /*
   * Laden und initialisieren von neuen Cube Maps, die von aussen gesetzt
   * wurden.
   */
  public void newCubeMaps()
  {
    lockNewTextures();
    for ( int i = 0; i < newCubeMapCount; i++ )
    {
      loadCubeMap( newCubeMapFilenames[i], newCubeMapTextureUnit[i] );
      newCubeMapFilenames[i] = null;
    }
    newCubeMapCount = 0;
    releaseNewTextures();
  }

  /*
   * Neue Textur von auen setzen
   */
  public void setTexture( String filename, int textureUnit )
  {
    lockNewTextures();
    if ( newTexturesCount < maxNewTexturesCount )
    {
      newTexturesFilename[newTexturesCount] = filename;
      newTexturesUnit[newTexturesCount] = textureUnit;
      newTexturesCount++;
    }
    releaseNewTextures();
  }

  /*
   * Laden und initialisieren von neuen Cube Maps, die von aussen gesetzt
   * wurden.
   */
  private void newTextures()
  {
    lockNewTextures();
    for ( int i = 0; i < newTexturesCount; i++ )
    {
      if ( newTexturesUnit[i] < glMaxTextures )
      {
        loadTexture( newTexturesFilename[i], newTexturesUnit[i] );
      }
      newTexturesFilename[i] = null;
    }
    newTexturesCount = 0;
    releaseNewTextures();
  }

  /*
   * Textur laden und aktivieren
   */
  private void loadTexture( String filename, int textureUnit )
  {
    texLoader.readTexture( filename );
    if ( texLoader.isOk() )
    {
      gl.glGenTextures( 1, textures[textureUnit] );
      gl.glBindTexture( GL_TEXTURE_2D, textures[textureUnit][0] );
      gl.glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
      gl.glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
      gl.glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB,
                       texLoader.getImageWidth(), texLoader.getImageHeight(),
                       0, GL_RGB, GL_UNSIGNED_BYTE,
                       texLoader.getTexture() );
      textureLoaded[textureUnit] = true;
      textureType[textureUnit] = GL_TEXTURE_2D;
    }
  }

  /*
   * Cube Map laden und aktivieren
   */
  public void loadCubeMap( String[] filenames, int textureUnit )
  {
    gl.glGenTextures( 1, textures[textureUnit] );
    gl.glBindTexture( GL_TEXTURE_CUBE_MAP_EXT, textures[textureUnit][0] );
    gl.glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );

    gl.glTexParameteri( GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    gl.glTexParameteri( GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    for ( int i = 0; i < 6; i++ )
    {
      texLoader.readTexture( filenames[i] );
      if ( texLoader.isOk() )
      {
        gl.glTexImage2D(
            cubefaces[i],
            0, // level
            GL_RGB, // internal format
            texLoader.getImageWidth(), // width
            texLoader.getImageHeight(), // height
            0, // border
            GL_RGB, // format
            GL_UNSIGNED_BYTE, // type
            texLoader.getTexture() ); // pixel data

        if ( cubefaces[i] == GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT )
        {
          gl.glGenTextures( 1, background_texture );
          gl.glBindTexture( GL_TEXTURE_2D, background_texture[0] );
          gl.glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
          gl.glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
          gl.glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB,
                           texLoader.getImageWidth(), texLoader.getImageHeight(),
                           0, GL_RGB, GL_UNSIGNED_BYTE,
                           texLoader.getTexture() );
          background = true;
        }
      }
    }
    textureLoaded[textureUnit] = true;
    textureType[textureUnit] = GL_TEXTURE_CUBE_MAP_EXT;
  }

  /*
   * Texturen aktivieren
   */
  public void bindTextures()
  {
    for ( int i = 0; i < glMaxTextures; i++ )
    {
      if ( textureLoaded[i] )
      {
        gl.glActiveTextureARB( units[i] );
        gl.glBindTexture( textureType[i], textures[i][0] );
        gl.glEnable( textureType[i] );
      }
    }
  }

  /*
   * Texturen deaktivieren
   */
  public void unBindTextures()
  {
    for ( int i = glMaxTextures - 1; i >= 0; i-- )
    {
      if ( textureLoaded[i] )
      {
        gl.glActiveTextureARB( units[i] );
        gl.glDisable( textureType[i] );
      }
    }
  }

  /*
   * Allgemeine Renderfunktion
   */
  public void display()
  {

    if ( glj.gljMakeCurrent() == false )
    {
      return;
    }

    gl.glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
    gl.glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    /*
     * Position der Lichtquelle festlegen
     */
    gl.glPushMatrix();
    gl.glRotatef( lightRotation, 0.0f, 1.0f, 0.0f );
    gl.glLightfv( GL_LIGHT0, GL_POSITION, light_position );
    lightRotation += lightRotOffset;
    gl.glPopMatrix();

    /*
     * Hintergrund, falls vorhanden, darstellen
     */
    if ( background )
    {
      gl.glDisable( GL_LIGHTING );
      gl.glEnable( GL_TEXTURE_2D );
      gl.glBindTexture( GL_TEXTURE_2D, background_texture[0] );

      gl.glPushMatrix();
      gl.glScalef( 4.0f, 4.0f, 1.0f );
      gl.glTranslatef( 0.0f, 0.0f, -1.5f );
      gl.glBegin( GL_QUADS );
      gl.glTexCoord2f( 1.0f, 1.0f );
      gl.glVertex3f( -1.0f, -1.0f, 0.0f );
      gl.glTexCoord2f( 0.0f, 1.0f );
      gl.glVertex3f( 1.0f, -1.0f, 0.0f );
      gl.glTexCoord2f( 0.0f, 0.0f );
      gl.glVertex3f( 1.0f, 1.0f, 0.0f );
      gl.glTexCoord2f( 1.0f, 0.0f );
      gl.glVertex3f( -1.0f, 1.0f, 0.0f );
      gl.glEnd();
      gl.glPopMatrix();
      gl.glDisable( GL_TEXTURE_2D );
      gl.glEnable( GL_LIGHTING );
    }

    /*
     * Materialeigenschaften des Testobjekts setzen
     */
    gl.glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, diffuseMaterial0 );
    gl.glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, specularMaterial0 );
    gl.glMateriali( GL_FRONT, GL_SHININESS, shininessMaterial0 );

    gl.glPushMatrix();
    gl.glRotatef( rotX, 1.0f, 0.0f, 0.0f );
    gl.glRotatef( rotY, 0.0f, 1.0f, 0.0f );

    /*
     * Texturen aktivieren
     */
    bindTextures();

    /*
     * Vertex- und Fragment-Programm aktivieren
     */
    if ( vertex )
    {
      cgGL.cgGLEnableProfile( CG_PROFILE_ARBVP1 );
    }
    if ( fragment )
    {
      cgGL.cgGLEnableProfile( CG_PROFILE_ARBFP1 );
    }

    /*
     * Objekt darstellen
     */
    switch ( objectIndex )
    {
      case 0:
        glut.glutSolidTeapot( 1.5 );
        break;
      case 1:
        gl.glPushMatrix();
        gl.glScalef( 1.25f, 1.25f, 1.25f );
        myCube( 25 );
        gl.glPopMatrix();
        break;
      case 2:
        gl.glPushMatrix();
        gl.glScalef( 1.7f, 1.7f, 1.7f );
        mySphere( 50 );
        gl.glPopMatrix();
        break;
    }
    cgGL.cgGLDisableProfile( CG_PROFILE_ARBVP1 );
    cgGL.cgGLDisableProfile( CG_PROFILE_ARBFP1 );

    unBindTextures();

    gl.glPopMatrix();

    newPrograms();
    newTextures();
    newCubeMaps();

    glj.gljSwap();
    glj.gljCheckGL();
    glj.gljFree();

  }

  /*
   * Multitexture-Wrfel
   */
  public void myCube( int tes )
  {
    float dx, dy, dz;

    float d = 2 / ( float ) tes;
    float dt = 1 / ( float ) tes;

    int param = 0;

    // front face
    if ( vertex )
    {
      param = cg.cgGetNamedParameter( vertexProgram, "t" );
      cgGL.cgGLSetParameter3fv( param, new float[]
                                {0.0f, 1.0f, 0.0f} );
    }
    gl.glNormal3f( 0.0f, 0.0f, 1.0f );
    for ( int i = 0; i < tes; i++ )
    {
      gl.glBegin( GL_QUAD_STRIP );
      for ( int j = 0; j <= tes; j++ )
      {
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], dt * ( float ) j, dt * ( float ) i );
        }
        gl.glVertex3f( -1.0f + ( float ) j * d, -1.0f + ( float ) i * d, 1.0f );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], dt * ( float ) j, dt * ( float ) ( i + 1 ) );
        }
        gl.glVertex3f( -1.0f + ( float ) j * d, -1.0f + ( ( float ) i + 1.0f ) * d, 1.0f );
      }
      gl.glEnd();
    }

    // back face
    if ( vertex )
    {
      cgGL.cgGLSetParameter3fv( param, new float[]
                                {0.0f, -1.0f, 0.0f} );
    }
    gl.glNormal3f( 0.0f, 0.0f, -1.0f );
    for ( int i = 0; i < tes; i++ )
    {
      gl.glBegin( GL_QUAD_STRIP );
      for ( int j = 0; j <= tes; j++ )
      {
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], 1.0f - dt * ( float ) j, dt * ( float ) i );
        }
        gl.glVertex3f( 1.0f - ( float ) j * d, 1.0f - ( float ) i * d, -1.0f );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], 1.0f - dt * ( float ) j, dt * ( float ) ( i + 1 ) );
        }
        gl.glVertex3f( 1.0f - ( float ) j * d, 1.0f - ( ( float ) i + 1.0f ) * d, -1.0f );
      }
      gl.glEnd();
    }

    // right face
    if ( vertex )
    {
      cgGL.cgGLSetParameter3fv( param, new float[]
                                {0.0f, 0.0f, 1.0f} );
    }
    gl.glNormal3f( 1.0f, 0.0f, 0.0f );
    for ( int i = 0; i < tes; i++ )
    {
      gl.glBegin( GL_QUAD_STRIP );
      for ( int j = 0; j <= tes; j++ )
      {
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], dt * ( float ) j, dt * ( float ) i );
        }
        gl.glVertex3f( 1.0f, -1.0f + ( float ) j * d, -1.0f + ( float ) i * d );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], dt * ( float ) j, dt * ( float ) ( i + 1 ) );
        }
        gl.glVertex3f( 1.0f, -1.0f + ( float ) j * d, -1.0f + ( float ) ( i + 1.0 ) * d );
      }
      gl.glEnd();
    }

    // left face
    if ( vertex )
    {
      cgGL.cgGLSetParameter3fv( param, new float[]
                                {0.0f, 0.0f, -1.0f} );
    }
    gl.glNormal3f( -1.0f, 0.0f, 0.0f );
    for ( int i = 0; i < tes; i++ )
    {
      gl.glBegin( GL_QUAD_STRIP );
      for ( int j = 0; j <= tes; j++ )
      {
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], 1.0f - dt * ( float ) j, dt * ( float ) i );
        }
        gl.glVertex3f( -1.0f, 1.0f - ( float ) j * d, 1.0f - ( float ) i * d );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], 1.0f - dt * ( float ) j, dt * ( float ) ( i + 1 ) );
        }
        gl.glVertex3f( -1.0f, 1.0f - ( float ) j * d, 1.0f - ( float ) ( i + 1.0 ) * d );
      }
      gl.glEnd();
    }

    // top face
    if ( vertex )
    {
      cgGL.cgGLSetParameter3fv( param, new float[]
                                {1.0f, 0.0f, 0.0f} );
    }
    gl.glNormal3f( 0.0f, 1.0f, 0.0f );
    for ( int i = 0; i < tes; i++ )
    {
      gl.glBegin( GL_QUAD_STRIP );
      for ( int j = 0; j <= tes; j++ )
      {
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], dt * ( float ) j, dt * ( float ) i );
        }
        gl.glVertex3f( -1.0f + ( float ) i * d, 1.0f, -1.0f + ( float ) j * d );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], dt * ( float ) j, dt * ( float ) ( i + 1 ) );
        }
        gl.glVertex3f( -1.0f + ( float ) ( i + 1 ) * d, 1.0f, -1.0f + ( float ) j * d );
      }
      gl.glEnd();
    }

    // bottom face
    if ( vertex )
    {
      cgGL.cgGLSetParameter3fv( param, new float[]
                                { -1.0f, 0.0f, 0.0f} );
    }
    gl.glNormal3f( 0.0f, -1.0f, 0.0f );
    for ( int i = 0; i < tes; i++ )
    {
      gl.glBegin( GL_QUAD_STRIP );
      for ( int j = 0; j <= tes; j++ )
      {
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], 1.0f - dt * ( float ) j, dt * ( float ) i );
        }
        gl.glVertex3f( 1.0f - ( float ) i * d, -1.0f, 1.0f - ( float ) j * d );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], 1.0f - dt * ( float ) j, dt * ( float ) ( i + 1 ) );
        }
        gl.glVertex3f( 1.0f - ( float ) ( i + 1 ) * d, -1.0f, 1.0f - ( float ) j * d );
      }
      gl.glEnd();
    }
  }

  float TWOPI = 2 * ( float ) Math.PI;
  float PID2 = ( float ) Math.PI / 2;

  /*
   * Multitexture-Kugel
   */
  void mySphere( int n )
  {


    int ux = 0;
    int uy = 1;
    int uz = 0;

    int i, j;
    float theta1, theta2, theta3;

    if ( n < 0 )
    {
      n = -n;
    }
    for ( j = 0; j < n / 2; j++ )
    {
      theta2 = ( j + 1 ) * TWOPI / ( n - 1 ) - PID2;
      theta1 = j * TWOPI / ( n - 1 ) - PID2;

      gl.glBegin( GL_QUAD_STRIP );
      for ( i = 0; i <= n; i++ )
      {
        theta3 = i * TWOPI / n;

        float ex = ( float ) Math.cos( theta2 ) * ( float ) Math.cos( theta3 );
        float ey = ( float ) Math.sin( theta2 );
        float ez = ( float ) Math.cos( theta2 ) * ( float ) Math.sin( theta3 );
        float px = ex;
        float py = ey;
        float pz = ez;

        gl.glNormal3f( ex, ey, ez );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], ( float ) i / ( float ) n,
                                   2 * ( ( float ) j + 1 ) / ( float ) n );
        }
        gl.glVertex3f( ex, ey, ez );

        ex = ( float ) Math.cos( theta1 ) * ( float ) Math.cos( theta3 );
        ey = ( float ) Math.sin( theta1 );
        ez = ( float ) Math.cos( theta1 ) * ( float ) Math.sin( theta3 );

        gl.glNormal3f( ex, ey, ez );
        for ( int k = 0; k < 8; k++ )
        {
          gl.glMultiTexCoord2fARB( units[k], ( float ) i / ( float ) n,
                                   2 * ( float ) j / ( float ) n );
        }
        gl.glVertex3f( ex, ey, ez );
      }
      gl.glEnd();
    }
  }

  public void mouseClicked( MouseEvent e )
  {
  }

  public void mouseEntered( MouseEvent e )
  {
  }

  public void mouseExited( MouseEvent e )
  {
  }

  public void mousePressed( MouseEvent e )
  {
    pressedX = e.getX();
    pressedRotX = rotX;
    pressedY = e.getY();
    pressedRotY = rotY;
  }

  public void mouseReleased( MouseEvent e )
  {
  }

  public void mouseDragged( MouseEvent e )
  {
    int newX = e.getX();
    int newY = e.getY();

    rotY = pressedRotY - ( ( float ) pressedX - ( float ) newX ) / 2.0f;
    rotX = pressedRotX + ( ( float ) pressedY - ( float ) newY ) / 2.0f;

  }

  public void mouseMoved( MouseEvent e )
  {
  }

  public void keyPressed( KeyEvent e)
  {
  }

  public void keyReleased( KeyEvent e)
  {
  }

  public void keyTyped( KeyEvent e)
  {
    char key = e.getKeyChar();

    if( key == 'c' )
    {
      if( objectIndex == 2) objectIndex = 0;
      else objectIndex++;
    }

    if( key == 'l' )
    {
      if( lightRotOffset == 1.6f ) lightRotOffset = 0;
      else lightRotOffset = 1.6f;
    }

  }

}