package vsl.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.w3c.dom.*;

import vsl.*;
import vsl.dataflow.*;
import vsl.lang.*;
import vsl.util.*;


public class Workspace extends JDesktopPane implements ActionListener
{

  JButton previewButton;
  JInternalFrame previewButtonFrame;

  PreviewInternalFrame previewFrame = null;

  DiagramCreator diagramCreator;
  ConstantCreator constantCreator;
  OperatorCreator operatorCreator;
  FunctionCreator functionCreator;
  ImportCreator importCreator;
  ExportCreator exportCreator;
  ConstructorCreator constructorCreator;
  StateCreator stateCreator;

  JInternalFrame toolBarFrame;
  JToolBar toolBar;
  JToggleButton showBlockCreator;
  JToggleButton showDiagramCreator;
  JToggleButton showConstantCreator;
  JToggleButton showFunctionCreator;
  JToggleButton showOperatorCreator;
  JToggleButton showImportCreator;
  JToggleButton showExportCreator;
  JToggleButton showConstructorCreator;
  JToggleButton showStateCreator;

  DiagramFrame diagramFrame = null;

  JInternalFrame textAreaFrame;
  JTextArea textArea;

  private ShaderTransformer transformer;

  public Workspace()
  {
    initWorkspace();
  }

  public void initWorkspace()
  {
    transformer = new ShaderTransformer( "xml/transform.xsl" );

    previewButtonFrame = new JInternalFrame( "Preview" );
    previewButtonFrame.setLayer( JLayeredPane.PALETTE_LAYER );
    previewButtonFrame.setVisible( true );
    previewButton = new JButton( "Preview" );
    previewButton.addActionListener( this );
    previewButton.setSize( 100, 20 );
    previewButton.setVisible( true );
    previewButtonFrame.getContentPane().add( previewButton );

    previewButtonFrame.pack();
    add( previewButtonFrame );

    createPreviewFrame();
    functionCreator = new FunctionCreator();
    functionCreator.addActionListener( this );
    functionCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( functionCreator );

    operatorCreator = new OperatorCreator();
    operatorCreator.addActionListener( this );
    operatorCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( operatorCreator );

    diagramCreator = new DiagramCreator();
    diagramCreator.addActionListener( this );
    diagramCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( diagramCreator );

    constantCreator = new ConstantCreator();
    constantCreator.addActionListener( this );
    constantCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( constantCreator );

    importCreator = new ImportCreator();
    importCreator.addActionListener( this );
    importCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( importCreator );

    exportCreator = new ExportCreator();
    exportCreator.addActionListener( this );
    exportCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( exportCreator );

    constructorCreator = new ConstructorCreator();
    constructorCreator.addActionListener( this );
    constructorCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( constructorCreator );

    stateCreator = new StateCreator();
    stateCreator.addActionListener( this );
    stateCreator.setLayer( JLayeredPane.PALETTE_LAYER );
    add( stateCreator );

    toolBarFrame = new JInternalFrame( "Add Components" );
    toolBar = new JToolBar();
    toolBar.setOrientation( JToolBar.HORIZONTAL );
    toolBarFrame.setContentPane( toolBar );

    // showBlockCreator = new JButton( new ImageIcon( "images/add_block.png" ) );
    // showBlockCreator.addActionListener( this );
    showDiagramCreator = new JToggleButton( "Diagram" );
    showDiagramCreator.addActionListener( this );
    showFunctionCreator = new JToggleButton( "Function" );
    showFunctionCreator.addActionListener( this );
    showOperatorCreator = new JToggleButton( "Operator" );
    showOperatorCreator.addActionListener( this );
    showConstantCreator = new JToggleButton( "Constant" );
    showConstantCreator.addActionListener( this );
    showImportCreator = new JToggleButton( "Import" );
    showImportCreator.addActionListener( this );
    showExportCreator = new JToggleButton( "Export" );
    showExportCreator.addActionListener( this );
    showConstructorCreator = new JToggleButton( "Constructor" );
    showConstructorCreator.addActionListener( this );
    showStateCreator = new JToggleButton( "GLState" );
    showStateCreator.addActionListener( this );

    // toolBar.add( showBlockCreator );
    toolBar.add( showDiagramCreator );
    toolBar.add( showFunctionCreator );
    toolBar.add( showOperatorCreator );
    toolBar.add( showConstructorCreator );
    toolBar.add( showStateCreator );
    toolBar.add( showConstantCreator );
    toolBar.add( showImportCreator );
    toolBar.add( showExportCreator );
    toolBarFrame.setVisible( true );
    toolBarFrame.setLayer( JLayeredPane.PALETTE_LAYER );
    toolBarFrame.pack();
    add( toolBarFrame );

    diagramFrame = new DiagramFrame();
    add( diagramFrame );
  }

  public DiagramFrame getDiagramFrame()
  {
    return diagramFrame;
  }

  public void createPreviewFrame()
  {
    previewFrame = new PreviewInternalFrame();
    previewFrame.setLayer( JLayeredPane.MODAL_LAYER );
    add( previewFrame );
  }

  public void setPreviewFrame( PreviewInternalFrame frame )
  {
    remove( previewFrame );
    previewFrame = frame;
    previewFrame.setLayer( JLayeredPane.MODAL_LAYER );
    add( previewFrame );
  }

  public PreviewInternalFrame getPreviewFrame()
  {
    return previewFrame;
  }

  public void actionPerformed( ActionEvent e )
  {
    Object source = e.getSource();
    if ( source == previewButton )
    {
      Document document = diagramFrame.getCanvas().getDocument();
      previewFrame.getPreview().setProgram( document );
    }
    if ( source == showDiagramCreator )
    {
      if ( diagramCreator.isVisible() == true )
      {
        diagramCreator.setVisible( false );
      }
      else
      {
        diagramCreator.setVisible( true );
      }
    }
    if ( source == showOperatorCreator )
    {
      if ( operatorCreator.isVisible() == true )
      {
        operatorCreator.setVisible( false );
      }
      else
      {
        operatorCreator.setVisible( true );
      }

    }
    if ( source == showConstructorCreator )
    {
      if ( constructorCreator.isVisible() == true )
      {
        constructorCreator.setVisible( false );
      }
      else
      {
        constructorCreator.setVisible( true );
      }
    }
    if ( source == showStateCreator )
    {
      if ( stateCreator.isVisible() == true )
      {
        stateCreator.setVisible( false );
      }
      else
      {
        stateCreator.setVisible( true );
      }
    }
    if ( source == showFunctionCreator )
    {
      if ( functionCreator.isVisible() == true )
      {
        functionCreator.setVisible( false );
      }
      else
      {
        functionCreator.setVisible( true );
      }
    }
    if ( source == showConstantCreator )
    {
      if ( constantCreator.isVisible() == true )
      {
        constantCreator.setVisible( false );
      }
      else
      {
        constantCreator.setVisible( true );
      }
    }
    if ( source == showImportCreator )
    {
      if ( importCreator.isVisible() == true )
      {
        importCreator.setVisible( false );
      }
      else
      {
        importCreator.setVisible( true );
      }

    }
    if ( source == showExportCreator )
    {
      if ( exportCreator.isVisible() == true )
      {
        exportCreator.setVisible( false );
      }
      else
      {
        exportCreator.setVisible( true );
      }

    }

    if ( source == functionCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        diagramFrame.getCanvas().addFunctionNode( functionCreator.getFunction() );
      }
    }
    if ( source == operatorCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        diagramFrame.getCanvas().addOperatorNode( operatorCreator.getOperator() );
      }
    }
    if ( source == constructorCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        diagramFrame.getCanvas().addConstructorNode( constructorCreator.getConstructor() );
      }
    }
    if ( source == stateCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        diagramFrame.getCanvas().addStateNode( stateCreator.getState() );
      }
    }
    if ( source == diagramCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        Diagram diagram = diagramCreator.getDiagram();
        if ( diagram.getCategoryInt() == Diagram.NEW )
        {
          String s = ( String ) JOptionPane.showInputDialog(
              this,
              "New Diagram Name:",
              "New Diagram",
              JOptionPane.QUESTION_MESSAGE );
          if ( s != null )
          {
            DataFlowDiagram dfDiagram = new DataFlowDiagram( diagram.getDocument() );
            dfDiagram.setName(s);
            diagramFrame.getCanvas().addDiagramNode(  dfDiagram );
          }
        }
        else{
          DataFlowDiagram dfDiagram = new DataFlowDiagram( diagram.getDocument() );
          diagramFrame.getCanvas().addDiagramNode( dfDiagram );
        }
      }
    }
    if ( source == constantCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        DataValue data = constantCreator.getDataObject();
        diagramFrame.getCanvas().addConstantNode( data );
      }
    }
    if ( source == importCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        if ( !importCreator.getName().equals( "" ) )
        {
          DataFlowSourceAnchor imp = importCreator.getImport();
          diagramFrame.getCanvas().addImport( imp );
        }
      }
    }

    if ( source == exportCreator.getAddButton() )
    {
      if ( diagramFrame != null )
      {
        if ( !exportCreator.getName().equals( "" ) )
        {
          DataFlowDestinationAnchor export = exportCreator.getExport();
          diagramFrame.getCanvas().addExport( export );
        }
      }
    }

  }

}