package vsl.lang;

/*
 * Objektreprsentation eines Datenwerts.
 */
public class DataValue
{

  /*
   * Speicherung der Daten
   */
  boolean[] booleanData;
  int[] intData;
  float[] floatData;
  double[] doubleData;

  /*
   * Datentyp
   */
  DataType type;

  /*
   * Stringreprsentation
   */
  String string;

  /*
   * Erzeugt einen neuen Datenwert.
   */
  public DataValue( DataType dataType )
  {
    this.type = dataType;
    switch ( dataType.getJavaType() )
    {
      case DataType.J_BOOLEAN:
        boolean[] booleanData = new boolean[dataType.rows * dataType.columns];
        for ( int i = 0; i < dataType.rows * dataType.columns; i++ )
        {
          booleanData[i] = true;
        }
        break;
      case DataType.J_INT:
        int[] intData = new int[dataType.rows * dataType.columns];
        for ( int i = 0; i < dataType.rows * dataType.columns; i++ )
        {
          intData[i] = 0;
        }
        break;
      case DataType.J_FLOAT:
        float[] floatData = new float[dataType.rows * dataType.columns];
        for ( int i = 0; i < dataType.rows * dataType.columns; i++ )
        {
          floatData[i] = 0.0f;
        }
        break;
      case DataType.J_DOUBLE:
        double[] doubleData = new double[dataType.rows * dataType.columns];
        for ( int i = 0; i < dataType.rows * dataType.columns; i++ )
        {
          doubleData[i] = 0.0;
        }
        break;
    }
  }

  /*
   * Erzeugt einen neuen Datenwert.
   */
  public DataValue( DataType dataType, float[] floatData )
  {
    this.type = dataType;
    this.floatData = floatData;
    string = createString();
  }

  /*
   * Erzeugt einen neuen Datenwert.
   */
  public DataValue( DataType dataType, boolean[] booleanData )
  {
    this.type = dataType;
    this.booleanData = booleanData;
    string = createString();
  }

  /*
   * Erzeugt einen neuen Datenwert.
   */
  public DataValue( DataType dataType, int[] intData )
  {
    this.type = dataType;
    this.intData = intData;
    string = createString();
  }

  /*
   * Erzeugt einen neuen Datenwert.
   */
  public DataValue( DataType dataType, double[] doubleData )
  {
    this.type = dataType;
    this.doubleData = doubleData;
    this.string = createString();
  }

  /*
   * Erzeugt einen neuen Datenwert.
   */
  public DataValue( DataType dataType, String dataString )
  {
    this.type = dataType;
    if ( type.getTypeClass() == DataType.ELEMENTAL )
    {
      switch ( type.getJavaType() )
      {
        case DataType.J_BOOLEAN:
          if ( dataString.equals( "true" ) )
          {
            booleanData = new boolean[]
                {
                true};
          }
          else
          {
            booleanData = new boolean[]
                {
                false};
          }
          break;
        case DataType.J_INT:
          intData = new int[]
              {
              Integer.parseInt( dataString )};
          break;
        case DataType.J_FLOAT:
          floatData = new float[]
              {
              Float.parseFloat( dataString )};
          break;
        case DataType.J_DOUBLE:
          doubleData = new double[]
              {
              Double.parseDouble( dataString )};
          break;
      }
    }
    else
    {
      switch ( type.getJavaType() )
      {
        case DataType.J_BOOLEAN:
          booleanData = new boolean[type.rows * type.columns];
          int firstIndex = dataString.indexOf( "(" );
          int secondIndex = dataString.indexOf( "," ) + 1;
          for ( int i = 0; i < type.rows * type.columns; i++ )
          {
            if ( dataString.substring( firstIndex, secondIndex ).equals( "true" ) )
            {
              booleanData[i] = true;
            }
            else
            {
              booleanData[i] = false;
            }
            firstIndex = secondIndex;
            if ( i != type.rows * type.columns - 1 )
            {
              secondIndex = dataString.indexOf( ",", firstIndex + 1 );
            }
            else
            {
              secondIndex = dataString.indexOf( ")", firstIndex + 1 );
            }
          }
          break;
        case DataType.J_FLOAT:
          floatData = new float[type.rows * type.columns];
          firstIndex = dataString.indexOf( "(" ) + 1;
          secondIndex = dataString.indexOf( "," );
          int n = type.rows * type.columns;
          for ( int i = 0; i < n; i++ )
          {
            floatData[i] = Float.parseFloat( dataString.substring( firstIndex, secondIndex ) );
            firstIndex = secondIndex + 1;
            if ( i == n - 2 )
            {
              secondIndex = dataString.indexOf( ")", firstIndex + 1 );
            }
            else
            {
              secondIndex = dataString.indexOf( ",", firstIndex + 1 );
            }
          }
          break;
      }

    }
    this.string = createString();
  }

  public String getValueAsString( int i )
  {
    switch ( type.getJavaType() )
    {
      case DataType.J_BOOLEAN:
        return Boolean.toString( booleanData[i] );
      case DataType.J_INT:
        return Integer.toString( intData[i] );
      case DataType.J_FLOAT:
        return Float.toString( floatData[i] );
      case DataType.J_DOUBLE:
        return Double.toString( doubleData[i] );
    }
    return null;
  }

  public float[] getFloatData()
  {
  	return floatData;
  }

  public void setData( boolean[] boolData )
  {
    this.booleanData = boolData;
    this.string = createString();
  }

  public void setData( int[] intData )
  {
    this.intData = intData;
    this.string = createString();
  }

  public void setData( float[] floatData )
  {
    this.floatData = floatData;
    this.string = createString();
  }

  public void setData( double[] doubleData )
  {
    this.doubleData = doubleData;
    this.string = createString();
  }

  public void setData( boolean boolValue, int i )
  {
    booleanData[i] = boolValue;
    this.string = createString();
  }

  public void setData( int intValue, int i )
  {
    intData[i] = intValue;
    this.string = createString();
  }

  public void setData( float floatValue, int i )
  {
    floatData[i] = floatValue;
    this.string = createString();
  }

  public void setData( double doubleValue, int i )
  {
    doubleData[i] = doubleValue;
    this.string = createString();
  }

  public DataType getType()
  {
    return type;
  }

  public String toString()
  {
    return string;
  }

  private String createString()
  {
    String tmp = new String();
    switch ( type.getJavaType() )
    {
      case DataType.J_BOOLEAN:
        if ( type.rows > 1 )
        {
          tmp = type.toString() + "(";
          for ( int i = 0; i < booleanData.length; i++ )
          {
            tmp += Boolean.toString( booleanData[i] );
            if ( i < booleanData.length - 1 )
            {
              tmp += ",";
            }
          }
          tmp += ")";
        }
        break;
      case DataType.J_INT:
        if ( type.rows > 1 )
        {
          tmp = type.toString() + "(";
          for ( int i = 0; i < intData.length; i++ )
          {
            tmp += Integer.toString( intData[i] );
            if ( i < intData.length - 1 )
            {
              tmp += ",";
            }
          }
          tmp += ")";
        }
        break;
      case DataType.J_FLOAT:
        if ( type.rows > 1 )
        {
          tmp = type.toString() + "(";
          for ( int i = 0; i < floatData.length; i++ )
          {
            tmp += Float.toString( floatData[i] );
            if ( i < floatData.length - 1 )
            {
              tmp += ",";
            }
          }
          tmp += ")";
        }
        else
        {
          tmp = Float.toString(floatData[0]);
        }
        break;
    }
    return tmp;
  }

}