package vsl.lang;

import java.awt.image.*;
import java.io.*;
import java.util.*;

import javax.imageio.ImageIO;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/*
 * Objektreprsentation einer OpenGL Zustands-Variable
 */
public class StateVariable
{

  /*
   * Name des Zustands
   */
  public String name;

  /*
   * Kurzer Name
   */
  public String shortName;

  /*
   * Datentyp
   */
  public DataType type;

  /*
   * Wert
   */
  public String value;

  /*
   * Kategorie
   */
  public int category;

  /*
   * Dateiname des Icons
   */
  public String iconFilename;

  /*
   * Icon
   */
  public BufferedImage icon;

  /*
   * Konstruktor
   */
  public StateVariable( String name, String shortName, DataType type, String value, int category,
                        String iconFilename )
  {
    this.name = name;
    this.shortName = shortName;
    this.type = type;
    this.value = value;
    this.category = category;
    this.iconFilename = iconFilename;
    try
    {
      icon = ImageIO.read( new File( iconFilename ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  /*
   * Gibt die Kategorie zurck.
   */
  public int getCategory()
  {
    return category;
  }

  /*
   * Gibt den Namen zurck.
   */
  public String getName()
  {
    return name;
  }

  /*
   * Gibt das Icon zurck.
   */
  public BufferedImage getIcon()
  {
    return icon;
  }

  /*
   * Gibt den Kurznamen zurck.
   */
  public String getShortName()
  {
    return shortName;
  }

  /*
   * Gibt das Datentyp zurck.
   */
  public DataType getType()
  {
    return type;
  }

  /*
   * Gibt den Wert zurck.
   */
  public String getValue()
  {
    return value;
  }

  /*
   * Ausgabe als String.
   */
  public String toString()
  {
    return name;
  }

  /*
   * Folgend ist ein statischer Teil, der den Zugriff auf untersttzte
   * Zustnde ermglicht.
   */

  /*
   * Kategorien
   */
  public static final int UNDEFINED = -1;
  public static final int MATRIX = 0;
  public static final int MATERIAL = 1;
  public static final int LIGHT = 2;
  public static final int TEXGEN = 3;

  /*
   * XML-Strings der Bereiche
   */
  public static final String[] cat_strings = new String[]
      {
      "matrix", "material", "light", "texgen"};

  public static final int catCount = 3;

  /*
   * Listen zur Speicherung
   */
  public static Vector matrix;
  public static Vector material;
  public static Vector light;
  public static Vector texgen;

  /*
   * Hashmap zur zustzlichen Speicherung
   */
  public static HashMap stateHashMap;

  /*
   * Initialisierung der Verwaltung
   */
  static
  {
    matrix = new Vector();
    material = new Vector();
    light = new Vector();
    texgen = new Vector();
    stateHashMap = new HashMap();
    importState( "xml/state.xml" );
  }

  /*
   * Gibt Kategorie nach Nummer i zurck.
   */
  public static String getCategory( int i )
  {
    return cat_strings[i];
  }

  /*
   * Gibt Kategorienummer zurck.
   */
  public static int getCategory( String category )
  {
    for ( int i = 0; i < cat_strings.length; i++ )
    {
      if ( cat_strings[i].equals( category ) )
      {
        return i;
      }
    }
    return UNDEFINED;
  }

  /*
   * Gibt Liste der Matrix-Zustnde zurck.
   */
  public static Vector getMatrixState()
  {
    return matrix;
  }

  /*
   * Gibt Liste der Material-Zustnde zurck.
   */
  public static Vector getMaterialState()
  {
    return material;
  }

  /*
   * Gibt Liste der Lichtquellen-Zustnde zurck.
   */
  public static Vector getLightState()
  {
    return light;
  }

  /*
   * Fgt eine Variable der Verwaltung zu.
   */
  public static void addVariable( StateVariable variable )
  {
    stateHashMap.put( variable.getName(), variable );
    switch ( variable.getCategory() )
    {
      case MATRIX:
        matrix.add( variable );
        break;
      case MATERIAL:
        material.add( variable );
        break;
      case LIGHT:
        light.add( variable );
        break;
    }
  }

  /*
   * XML-Import
   */
  public static void importState( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList variablesListXML = rootElement.getElementsByTagName( "Variable" );
      for ( int i = 0; i < variablesListXML.getLength(); i++ )
      {
        Element variableXML = ( Element ) ( variablesListXML.item( i ) );
        String variableName = variableXML.getAttribute( "name" );
        String variableShortName = variableXML.getAttribute( "short_name" );
        DataType variableType = DataType.getDataType( variableXML.getAttribute( "type" ) );
        String variableValue = variableXML.getAttribute( "value" );
        int variableCategory = getCategory( variableXML.getAttribute( "category" ) );
        String iconFilename = variableXML.getAttribute( "icon" );
        StateVariable variable = new StateVariable( variableName, variableShortName, variableType,
            variableValue, variableCategory, iconFilename );
        addVariable( variable );
      }
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }

  }

  public static StateVariable getVariable( String name )
  {
    return ( StateVariable ) stateHashMap.get( name );
  }

}