package vsl.lang;

import java.net.URL;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/*
 * Objektreprsentation eines zusammengesetzten Datentyps.
 */
public class Struct
{

  public static final int UNDEFINED = -1;
  public static final int VERTEX_IN = 0;
  public static final int VERTEX_OUT = 1;
  public static final int FRAGMENT_IN = 2;
  public static final int FRAGMENT_OUT = 3;

  protected Document structDocument;
  protected Element structRoot;

  protected String name;
  protected int category;

  private NodeList inputList = null;
  private NodeList outputList = null;

  public Struct( String documentFileName ) throws
      SAXException, SAXParseException, IOException
  {

    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      structDocument = builder.parse( documentFileName );
      init();
    }
    catch ( ParserConfigurationException e )
    {
      System.err.println( e );
    }
  }

  private void init()
  {
    structRoot = structDocument.getDocumentElement();
    name = structRoot.getAttribute( "name" );
    category = getCategory( structRoot.getAttribute( "category" ) );
    inputList = structRoot.getElementsByTagName( "Input" );
    outputList = structRoot.getElementsByTagName( "Output" );
  }

  public int getCategory()
  {
    return category;
  }

  public String getName()
  {
    return name;
  }

  public int getInputCount()
  {
    return inputList.getLength();
  }

  public String getInputName( int i )
  {
    Element e = ( Element ) inputList.item( i );
    return e.getAttribute( "name" );
  }

  public String getInputType( int i )
  {
    Element e = ( Element ) inputList.item( i );
    return e.getAttribute( "type" );
  }

  public String getInputDefault( int i )
  {
    Element e = ( Element ) inputList.item( i );
    return e.getAttribute( "default" );
  }

  public int getOutputCount()
  {
    return outputList.getLength();
  }

  public String getOutputName( int i )
  {
    Element e = ( Element ) outputList.item( i );
    return e.getAttribute( "name" );
  }

  public String getOutputType( int i )
  {
    Element e = ( Element ) outputList.item( i );
    return e.getAttribute( "type" );
  }

  public String toString()
  {
    return name;
  }


  /*
   * Folgend ist ein statischer Teil, der den Zugriff auf die untersttzten
   * Structs ermglicht. Diese werden aus einer XML-Datei eingelesen.
   */

  /*
   * Speicherung der Structs nach Einordung getrennt
   */
  public static Vector vertex_in;
  public static Vector vertex_out;
  public static Vector fragment_in;
  public static Vector fragment_out;

  /*
   * Hashmap fr die schnelle Suche nach Namen
   */
  public static HashMap structsHashMap;

  /*
   * Initialierung und Einlesen der Daten
   */
  static
  {
    vertex_in = new Vector();
    vertex_out = new Vector();
    fragment_in = new Vector();
    fragment_out = new Vector();
    structsHashMap = new HashMap();
    importStructs( "xml/structs.xml" );
  }

  public static int getCategory( String catString )
  {
    if ( catString.equals( "vertex_in" ) )
    {
      return VERTEX_IN;
    }
    if ( catString.equals( "vertex_out" ) )
    {
      return VERTEX_OUT;
    }
    if ( catString.equals( "fragment_in" ) )
    {
      return FRAGMENT_IN;
    }
    if ( catString.equals( "fragment_out" ) )
    {
      return FRAGMENT_OUT;
    }
    return UNDEFINED;
  }

  /*
   * Einlesen der untersttzten Structs aus einer XML-Datei.
   */
  public static void importStructs( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList blockListXML = rootElement.getElementsByTagName( "Struct" );
      for ( int i = 0; i < blockListXML.getLength(); i++ )
      {
        Element blockXML = ( Element ) ( blockListXML.item( i ) );
        Struct struct = new Struct( blockXML.getAttribute( "filename" ) );
        addStruct( struct );
      }
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }

  /*
   * Struct hinzufgen
   */
  public static void addStruct( Struct struct )
  {
    switch ( struct.getCategory() )
    {
      case VERTEX_IN:
        vertex_in.add( struct );
        break;
      case VERTEX_OUT:
        vertex_out.add( struct );
        break;
      case FRAGMENT_IN:
        fragment_in.add( struct );
        break;
      case FRAGMENT_OUT:
        fragment_out.add( struct );
        break;
    }
    String name = struct.getName();
    structsHashMap.put( struct.getName(), struct );
  }

  public static Struct getStruct( String name )
  {
    if ( structsHashMap.containsKey( name ) )
    {
      return ( Struct ) structsHashMap.get( name );
    }
    else
    {
      return null;
    }
  }

  public static Vector getVertexIn()
  {
    return vertex_in;
  }

  public static Vector getVertexOut()
  {
    return vertex_out;
  }

  public static Vector getFragmentIn()
  {
    return fragment_in;
  }

  public static Vector getFragmentOut()
  {
    return fragment_out;
  }

}