<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="text" indent="no" />

<!-- builds the main structure of a vertex program -->
<xsl:template match="VertexProgram">

<xsl:apply-templates select="InputStruct" mode="declare" />

<xsl:apply-templates select="OutputStruct" mode="declare" />

<xsl:apply-templates select="Node" mode="declare" />

<xsl:value-of select="OutputStruct/@type" /> main(
<xsl:apply-templates select="InputStruct" mode="dontknow" />
<xsl:apply-templates select="Input" />)
{
<xsl:apply-templates select="OutputStruct" mode="dontknow" />

<xsl:apply-templates select="Node[position()=1]" mode="call" />
<xsl:apply-templates select="Export" />

return <xsl:value-of select="OutputStruct/@name" />;
}
</xsl:template>

<!-- builds the main structure of a fragment program -->
<xsl:template match="FragmentProgram">
<xsl:apply-templates select="InputStruct" mode="declare" />

<xsl:apply-templates select="OutputStruct" mode="declare" />

<xsl:apply-templates select="Node" mode="declare" />

<xsl:value-of select="OutputStruct/@type" /> main(
<xsl:apply-templates select="InputStruct" mode="dontknow" />
<xsl:apply-templates select="Input" />)
{
<xsl:apply-templates select="OutputStruct" mode="dontknow" />

<xsl:apply-templates select="Node[position()=1]" mode="call" />
<xsl:apply-templates select="Export" />

return <xsl:value-of select="OutputStruct/@name" />;
}
</xsl:template>

<!-- -->
<xsl:template match="InputStruct" mode="dontknow">
<xsl:value-of select="concat(@type,' ',@name)" />
</xsl:template>

<xsl:template match="OutputStruct" mode="dontknow">
<xsl:value-of select="concat(@type,' ',@name,';')" />
</xsl:template>

<!-- declaration of the input struct of a program -->
<xsl:template match="InputStruct" mode="declare">
<xsl:variable name="filename">
<xsl:value-of select="concat('structs/',@type,'.xml')" />
</xsl:variable>
<xsl:apply-templates select="document($filename)//Struct" mode="declare" />
</xsl:template>

<xsl:template match="Struct/Input" mode="declare">
<xsl:value-of select="concat(@type,' ',@name,' : ',@bind,';&#xA;')" />
</xsl:template>

<xsl:template match="Struct" mode="declare">
<xsl:value-of select="string('struct ')" />
<xsl:value-of select="@name" />
<xsl:value-of select="string('{&#xA;')" />
<xsl:apply-templates select="Input" mode="declare" />
<xsl:apply-templates select="Output" mode="declare" />
<xsl:value-of select="string('};&#xA;')" />
</xsl:template>

<!-- declaration of the output struct of a program -->
<xsl:template match="OutputStruct" mode="declare">
<xsl:variable name="filename">
<xsl:value-of select="concat('structs/',@type,'.xml')" />
</xsl:variable>
<xsl:apply-templates select="document($filename)//Struct" mode="declare" />
</xsl:template>

<xsl:template match="Struct/Output" mode="declare">
<xsl:value-of select="concat(@type,' ',@name,' : ',@bind,';&#xA;')" />
</xsl:template>

<!-- input parameter of vertex program -->
<xsl:template match="VertexProgram/Input">
<xsl:value-of select="concat(',&#xA;','uniform ',@type,' ',@name)" />
</xsl:template>

<!-- input parameter of fragment program -->
<xsl:template match="FragmentProgram/Input">
<xsl:value-of select="concat(',&#xA;','uniform ',@type,' ',@name)" />
</xsl:template>

<!-- assignment of program's output -->
<xsl:template match="Export">
<xsl:value-of select="concat(@name,' = ')" />
<xsl:apply-templates select="Connection" mode="export" />
<xsl:apply-templates select="Import" mode="getValue" />
<xsl:apply-templates select="Constant" mode="getValue" />
<xsl:value-of select="string(';&#xA;')" />
</xsl:template>

<!-- diagrams of a diagram node must be declared -->
<xsl:template match="Node[@type='diagram']" mode="declare">
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:apply-templates select="Diagram" mode="declare">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
</xsl:template>

<!-- declaration of diagrams (as functions) -->
<xsl:template match="Diagram" mode="declare">
<xsl:param name="functionCallID" />
<xsl:apply-templates select="Node" mode="declare" />
<xsl:value-of select="concat('void ',../@name,$functionCallID,'( ')" />
<xsl:apply-templates select="Output" mode="declare" />
<xsl:apply-templates select="Input" mode="declare" />
<xsl:value-of select="string(')&#xA;{&#xA;')" />
<xsl:apply-templates select="Node[position()=1]" mode="call" />
<xsl:apply-templates select="Export" />
<xsl:value-of select="string('}&#xA;&#xA;')" />
</xsl:template>

<!-- function output parameters -->
<xsl:template match="Output" mode="declare">
<xsl:if test="position()!=1"><xsl:value-of select="string(',')" /></xsl:if>
<xsl:value-of select="concat('out ', @type, ' ', @name)" />
</xsl:template>

<!-- function input parameters -->
<xsl:template match="Input" mode="declare">
<xsl:value-of select="concat(', in ', @type, ' ', @name)" />
</xsl:template>

<!-- generation of function call (diagram nodes) -->
<xsl:template match="Node[@type='diagram']" mode="call">
<xsl:apply-templates select="following-sibling::Node[position()=1]" mode="call" />
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:apply-templates select="Diagram/Output" mode="declareOutputVariables">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
<xsl:apply-templates select="." mode="functionCall">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
</xsl:template>

<!--  -->
<xsl:template match="Node[@type='operator']" mode="call">
<xsl:apply-templates select="following-sibling::Node[position()=1]" mode="call" />
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:value-of select="concat( @result, ' ')" />
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(';&#xA;')" />
<xsl:apply-templates select="." mode="functionCall">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
</xsl:template>

<!-- generation of function call (function nodes) -->
<xsl:template match="Node[@type='function']" mode="call">
<xsl:apply-templates select="following-sibling::Node[position()=1]" mode="call" />
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:value-of select="concat( @result, ' ')" />
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(';&#xA;')" />
<xsl:apply-templates select="." mode="functionCall">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
</xsl:template>

<!-- -->
<xsl:template match="Node[@type='constructor']" mode="call">
<xsl:apply-templates select="following-sibling::Node[position()=1]" mode="call" />
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:value-of select="concat( @result, ' ')" />
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(';&#xA;')" />
<xsl:apply-templates select="." mode="functionCall">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
</xsl:template>

<!-- insertion of state variable -->
<xsl:template match="Node[@type='state']" mode="call">
<xsl:apply-templates select="following-sibling::Node[position()=1]" mode="call" />
</xsl:template>

<xsl:template match="Node[@type='constant']" mode="call">
<xsl:apply-templates select="following-sibling::Node[position()=1]" mode="call" />
</xsl:template>

<!--
Ausgabeparameter eines Knoten mssen vorher deklariert werden
-->
<xsl:template match="Output" mode="declareOutputVariables">
<xsl:param name="functionCallID" />
<xsl:value-of select="concat( @type, ' ', @name, $functionCallID, ';&#xA;')" />
</xsl:template>

<!-- the function call -->
<xsl:template match="Node[@type='diagram']" mode="functionCall">
<xsl:param name="functionCallID" />
<xsl:value-of select="@name" />
<xsl:value-of select="$functionCallID" />
<xsl:value-of select="string('(')" />
<xsl:apply-templates select="Diagram/Output" mode="functionCall">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
<xsl:apply-templates select="Data" mode="functionCall" />
<xsl:value-of select="string(');&#xA;')" />
</xsl:template>

<!-- the function call -->
<xsl:template match="Node[@type='function']" mode="functionCall">
<xsl:param name="functionCallID" />
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="concat(' = ', @name, '(')" />
<xsl:apply-templates select="Data" mode="functionCall2" />
<xsl:value-of select="string(');&#xA;')" />
</xsl:template>

<!-- the function call -->
<xsl:template match="Node[@type='constructor']" mode="functionCall">
<xsl:param name="functionCallID" />
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(' = ')" />
<xsl:variable name="name" select="@name" />
<xsl:apply-templates select="document('constructors.xml')/Constructors/Constructor[@name=$name]" mode="getType" />
<xsl:value-of select="string('(')" />
<xsl:apply-templates select="Data" mode="getSwizzles">
<xsl:with-param name="constructor_name" select="@name" />
</xsl:apply-templates>
<xsl:value-of select="string(')')" />
<xsl:value-of select="string(';&#xA;')" />
</xsl:template>

<!--  -->
<xsl:template match="Node[@type='operator'][@op_type='infix']" mode="functionCall">
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(' = ')" />
<xsl:apply-templates select="Data[position()=1]" mode="getOutput" />
<xsl:value-of select="@symbol" />
<xsl:apply-templates select="Data[position()=2]" mode="getOutput" />
<xsl:value-of select="string(';&#xA;')" />
</xsl:template>

<!-- the function call -->
<xsl:template match="Node[@type='operator'][@op_type='praefix']" mode="functionCall">
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(' = ')" />
<xsl:value-of select="@symbol" /><xsl:apply-templates select="Data[position()=1]" mode="getOutput" />
<xsl:value-of select="string(';&#xA;')" />
</xsl:template>

<xsl:template match="Node[@type='operator'][@op_type='postfix']" mode="functionCall">
<xsl:apply-templates select="." mode="getOutput" />
<xsl:value-of select="string(' = ')" />
<xsl:apply-templates select="Data[position()=1]" mode="getOutput" /><xsl:value-of select="@symbol" />
<xsl:value-of select="string(';&#xA;')" />
</xsl:template>


<!-- output parameter of function call -->
<xsl:template match="Output" mode="functionCall">
<xsl:param name="functionCallID" />
<xsl:if test="position()!=1"><xsl:value-of select="string(',')" /></xsl:if>
<xsl:value-of select="concat( @name, $functionCallID )" />
</xsl:template>

<!-- input parameter of function call -->
<xsl:template match="Data" mode="functionCall">
<xsl:value-of select="string(',')" />
<xsl:apply-templates select="Connection" mode="getOutput" />
<xsl:apply-templates select="Import" mode="getValue" />
<xsl:apply-templates select="Default" mode="getValue" />
</xsl:template>

<xsl:template match="Data" mode="functionCall2">
<xsl:if test="position()!=1"><xsl:value-of select="string(',')" /></xsl:if>
<xsl:apply-templates select="Connection" mode="getOutput" />
<xsl:apply-templates select="Import" mode="getValue" />
<xsl:apply-templates select="Constant" mode="getValue" />
</xsl:template>

<xsl:template match="Data" mode="getOutput">
<xsl:apply-templates select="Connection" mode="getOutput" />
<xsl:apply-templates select="Import" mode="getValue" />
<xsl:apply-templates select="Default" mode="getValue" />
</xsl:template>

<!-- get default value -->
<xsl:template match="Default" mode="getValue">
<xsl:value-of select="@value" />
</xsl:template>

<!-- get value from import -->
<xsl:template match="Import" mode="getValue">
<xsl:value-of select="@name" />
</xsl:template>

<!-- get output from connected node -->
<xsl:template match="Connection" mode="getOutput">
<xsl:variable name="source" select="@source" />
<xsl:variable name="id" select="@sourceid" />
<xsl:apply-templates select="../../../Node[@name=$source][@id=$id]" mode="getOutput">
<xsl:with-param name="output" select="@output" />
</xsl:apply-templates>
</xsl:template>

<!-- get output from connected node -->
<xsl:template match="Connection" mode="export">
<xsl:variable name="source" select="@source" />
<xsl:variable name="id" select="@sourceid" />
<xsl:apply-templates select="../../Node[@name=$source][@id=$id]" mode="getOutput">
<xsl:with-param name="output" select="@output" />
</xsl:apply-templates>
</xsl:template>

<!-- get output from state variable node -->
<xsl:template match="Node[@type='state']" mode="getOutput">
<xsl:value-of select="@value" />
</xsl:template>

<!-- get output from constant node -->
<xsl:template match="Node[@type='constant']" mode="getOutput">
<xsl:value-of select="@value" />
</xsl:template>

<!-- get type from combinder node -->
<xsl:template match="Constructor" mode="getType">
<xsl:value-of select="@type" />
</xsl:template>

<!-- *i forgot* -->
<xsl:template match="Node[@type='constructor']/Data" mode="getSwizzles">
<xsl:param name="constructor_name" />
<xsl:variable name="input_name" select="@name" />
<xsl:variable name="input_value">
<xsl:apply-templates select="." mode="getOutput" />
</xsl:variable>
<xsl:if test="position()!=1">
<xsl:value-of select="string(',')" />
</xsl:if>
<xsl:apply-templates select="document('constructors.xml')/Constructors/Constructor[@name=$constructor_name]/Swizzle[@input=$input_name]" mode="getInput">
<xsl:with-param name="input_value" select="$input_value" />
</xsl:apply-templates>
</xsl:template >

<!-- -->
<xsl:template match="Swizzle" mode="getInput">
<xsl:param name="input_value" />
<xsl:if test="position()!=1">
<xsl:value-of select="string(',')" />
</xsl:if>
<xsl:value-of select="concat($input_value,@swizzle)" />
</xsl:template>

<!-- get output from diagram node -->
<xsl:template match="Node[@type='diagram']" mode="getOutput">
<xsl:param name="output" />
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:apply-templates select="Diagram/Output[@name=$output]" mode="getOutput">
<xsl:with-param name="functionCallID" select="$functionCallID" />
</xsl:apply-templates>
</xsl:template>

<!-- get output from function node -->
<xsl:template match="Node[@type='function']" mode="getOutput">
<xsl:value-of select="concat( @name, generate-id(), @id )" />
</xsl:template>

<!-- get output from operator node -->
<xsl:template match="Node[@type='operator']" mode="getOutput">
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:value-of select="concat( 'op', $functionCallID )" />
</xsl:template>

<!-- get output from constructor node -->
<xsl:template match="Node[@type='constructor']" mode="getOutput">
<xsl:variable name="functionCallID">
<xsl:value-of select="concat( generate-id(), @id )" />
</xsl:variable>
<xsl:value-of select="concat( 'comb', $functionCallID )" />
</xsl:template>

<!--  -->
<xsl:template match="Output" mode="getOutput">
<xsl:param name="functionCallID" />
<xsl:value-of select="concat(@name,$functionCallID)" />
</xsl:template>

<xsl:template match="Output" mode="functionCall">
<xsl:param name="functionCallID" />
<xsl:if test="position()!=1"><xsl:value-of select="string(',')" /></xsl:if>
<xsl:value-of select="concat(@name,$functionCallID)" />
</xsl:template>

</xsl:stylesheet>